<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\API\BaseController as BaseController;
use Illuminate\Http\Request;
use App\Models\Product;
use App\Models\Chat;
use App\Models\ChatMessage;
use Redirect;
use Auth;
use Log;

class ChatController extends BaseController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        // $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('home');
    }

    public function chatList(){
        $chat_list                                      = Chat::withCount('unreadedMessages')->where('creator' , Auth::user()->id)->orderBy('updated_at','desc')->get();
        if($chat_list->isEmpty()){
            return $this->sendError('No Data Found'); 
        }else{
            $success                                    = ['status' => true];
            if($chat_list){
                foreach($chat_list as $chat) {
                    $temp_data                          = [];
                    $temp_data                          = [
                                                            'date'              => date('d-m-Y', strtotime($chat->date)),
                                                            'status'            => $chat->status,
                                                            'scanned_code'      => $chat->code,
                                                            ];
                    if($chat->product){
                        $temp_data['product_name']      = $chat->product->product_name;
                        $temp_data['product_image']     = 'storage/app/images/'.$chat->product->product_image;
                    }
                    $temp_data['unreaded_msg_count']    = sizeof($chat->unreadedMessages);
                    if($chat->lastMessage){
                        $temp_data['last_message']      = $chat->lastMessage[0]->message;
                    }else{
                        $temp_data['last_message']      = '';
                    }
                    $success['product_list'][]          =  $temp_data;  
                }
            }
            echo json_encode($success);
            return ;
        }
    }

    public function chatDetails(Request $request){
        $scanned_code                                   = $request->input('code');
        $deooded_code                                   = explode('-', $scanned_code );
        if(sizeof($deooded_code) < 4){
            return $this->sendError('No Data Found'); 
        }
        $chat_details                                   = Chat::where(['creator' => Auth::user()->id, 'code' => $scanned_code,'product_id' => $deooded_code[2]])->first();
        if(!$chat_details){
            return $this->sendError('No Data Found'); 
        }else{
            $success                                    = ['status' => true];
            $messages                                   = [];
            if($chat_details->messages->isNotEmpty()){
                foreach($chat_details->messages as $chat) {
                    $temp_chat_message                  = ChatMessage::where('type', 1)->find($chat->id);
                    if($temp_chat_message){
                        $temp_chat_message->status      = 0;
                        $temp_chat_message->save();
                    }

                    $temp_data                          = [
                                                            'date'      => date('d-m-Y h:i:s A', strtotime($chat->date)),
                                                            'message'   => $chat->message,
                                                            'type'      => $chat->type,
                                                            'status'    => $chat->status,
                                                            ];
                    if($chat->chat_file != ''){
                        $temp_data['chat_file']         =  'storage/app/chat_files/'.$chat->chat_file; 
                    }else{
                        $temp_data['chat_file']         = '';
                    }
                    $messages[]                         = $temp_data;
                }
            }
            $success['messages']                        = $messages;
            echo json_encode($success);
            return ;
        }
    }
    /*
        public function chatCreate(Request $request){
            $new_chat               = new Chat();
            $new_chat->date         = date('y-m-d H:i:s');
            $new_chat->creator      = Auth::user()->id;
            $new_chat->receiver     = 1;
            $new_chat->title        = $request->input('title');
            $new_chat->status       = 1;
            if($new_chat->save()){
                $success            = [];
                $success['status']  = true;
                $success['title']   = $new_chat->title;
                $success['chat_id'] = $new_chat->id;
                $success['message'] = 'New Chat Created Successfully!';
                echo json_encode($success);
                return;
            }else{
                return $this->sendError('Some thing Went Wrong!'); 
            }
        }   
    */
    public function chatMessage(Request $request){
        if($request->input('messageFile')){
            $validator                                  = Validator::make($request->all(), [
                                                                'messageFile' => 'required|mimes:jpeg,jpg,png,gif|max:10000'
                                                            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        } 
        $scanned_code                                   = $request->input('code');
        $deooded_code                                   = explode('-', $scanned_code );
        $chat_details                                   = Chat::where(['creator' => Auth::user()->id, 'code' => $scanned_code,'product_id' => $deooded_code[2]])->first();
        if(!$chat_details){
            $product_details                            = Product::find($deooded_code[2]);
            $chat_details                               = new Chat();
            $chat_details->date                         = date('y-m-d H:i:s');
            $chat_details->creator                      = Auth::user()->id;
            $chat_details->receiver                     = 1;
            $chat_details->title                        = $request->input('title');
            $chat_details->product_id                   = $deooded_code[2];
            $chat_details->company_id                   = $product_details->company_id;
            $chat_details->code                         = $scanned_code;
            $chat_details->chat_type                    = $request->input('chat_type');
            $chat_details->chat_on                      = $request->input('chat_on');
            $chat_details->status                       = 1;
            $chat_details->save();
        }
        $new_message                                    = new ChatMessage();
        $new_message->chat_id                           = $chat_details->id;
        $new_message->date                              = date('y-m-d H:i:s');
        $new_message->status                            = 1;
        $new_message->message                           = $request->input('message');
        
        if($request->hasFile('messageFile')){
            $chat_file_file                             = time().Auth::user()->id.'.'.$request->messageFile->extension();
            if($request->messageFile->storeAs('public/app/chat_files', $chat_file_file)){
                $new_message->chat_file                = $chat_file_file;
            }
        }
        
        $new_message->user_id                           = Auth::user()->id;
        $new_message->type                              = 0;
        if($new_message->save()){
            $chat_details->updated_at                   = date('Y-m-d H:i:s');
            $chat_details->save();
            $success                                    = [];
            $success['status']                          = true;
            $success['message']                         = $new_message->message;
            $success['message_id']                      = $new_message->id;
            $success['message']                         = 'New message Created Successfully!';
            echo json_encode($success);
            return;
        }else{      
            return $this->sendError('Some thing Went Wrong!'); 
        } 
    }    
    public function details($product_code){
        return Redirect::away('https://www.google.com/');
    }
}