<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\API\BaseController as BaseController;
use Illuminate\Http\Request;
use App\Models\AppProductWarranty;
use App\Models\AppWarrantyPart;
use App\Models\AppExtendedWarranty;
use App\Models\Product;
use Validator;
use Redirect;
use Auth;
use Log;

class WarrantyController extends BaseController
{
    /**
     * Create a new controller instance.
     *
     * @return void
    */
    public function __construct()
    {
        // $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
    */
    
    public function newWarranty(Request $request){
        $validator = Validator::make($request->all(), [
            'product_name'          => 'required',
            'product_brand'         => 'required',
            'service'               => 'required',
            'whole_product'         => 'required',
            'waranty_period'        => 'required',
            'purchase_point_name'   => 'required',
            'purchase_point_phone'  => 'required',
            'service_center_name'   => 'required',
            'service_center_phone'  => 'required',
            'invoice_date'          => 'required',
        ]);
        
        if($validator->fails()){
            return $this->sendError('Validation Error.', $validator->errors());       
        }        

        if($request->input('invoice_file')){
            $validator = Validator::make($request->all(), [
                'invoice_file'         => 'required|max:10000'
            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        if($request->input('service_invoice_file')){
            $validator = Validator::make($request->all(), [
                'service_invoice_file'         => 'required|max:10000'
            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        if($request->input('product_image')){
            $validator = Validator::make($request->all(), [
                'product_image'         => 'required|mimes:jpeg,jpg,png,gif|max:10000'
            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        if($request->input('extended ') == 1){
            $validator = Validator::make($request->all(), [
                'app_warranty_id'       => 'required',
                'warranty_period'       => 'required',
                'provider_name'         => 'required',
                'provider_phone'        => 'required',
            ]);       
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        $scanned_code                           = $request->input('scan_code');
        if($scanned_code != ''){
            $temp_product_warranty              = AppProductWarranty::where(['user_id' => Auth::user()->id, 'scan_id' => $scanned_code])->first();
            if($temp_product_warranty){
                return $this->sendError('Warranty Already Added!. Please check'); 
            }
            $deooded_code                       = explode('-', $scanned_code );
            $productDetails                     = Product::find($deooded_code[2]);            
        }

        $new_warranty                           = new AppProductWarranty();
        $new_warranty->scan_id                  = $request->input('scan_code');
        $new_warranty->user_id                  = Auth::user()->id;
        if(isset($productDetails)){
            $new_warranty->product_id           = $productDetails->id;
            $new_warranty->product_name         = $productDetails->product_name;
            $new_warranty->product_brand        = $productDetails->brand_name;
        }else{
            $new_warranty->product_name         = $request->input('product_name');
            $new_warranty->product_brand        = $request->input('product_brand');
            if($request->hasFile('product_image')){
                $product_image_file             = time().Auth::user()->id.'PI.'.$request->product_image->extension();
                if($request->product_image->storeAs('public/app/warranty_files', $product_image_file)){
                    $new_warranty->product_image = $product_image_file;
                }
            }
        }
        $new_warranty->type                     = $request->input('service');
        $new_warranty->product_warranty         = $request->input('whole_product');
        $new_warranty->waranty_period           = $request->input('waranty_period');
        $new_warranty->purchase_point_name      = $request->input('purchase_point_name');
        $new_warranty->purchase_point_phone     = $request->input('purchase_point_phone');
        $new_warranty->service_center_name      = $request->input('service_center_name');
        $new_warranty->service_center_phone     = $request->input('service_center_phone');
        $new_warranty->invoice_date             = date('Y-m-d H:i:s', strtotime($request->input('invoice_date')));
        $new_warranty->note                     = $request->input('note');
        $new_warranty->registration_type        = 0;

        if($request->hasFile('invoice_file')){
            $product_invoice_file               = time().Auth::user()->id.'IF.'.$request->invoice_file->extension();
            if($request->invoice_file->storeAs('public/app/warranty_files', $product_invoice_file)){
                $new_warranty->invoice_file     = $product_invoice_file;
            }
        }

        if($request->hasFile('service_invoice_file')){
            $product_service_invoice_file       = time().Auth::user()->id.'SIF.'.$request->service_invoice_file->extension();
            if($request->service_invoice_file->storeAs('public/app/warranty_files', $product_service_invoice_file)){
                $new_warranty->service_invoice_file = $product_service_invoice_file;
            }
        }
        
        if($new_warranty->type == 0){ //service_warranty
            if($new_warranty->product_warranty == 0){ //whole_product_warranty
                $new_warranty->waranty_period   = $request->input('waranty_period');
            }
        }else{ // replacement
            $new_warranty->waranty_period       = $request->input('waranty_period');
        }

        if($new_warranty->save()){
            if($new_warranty->type == 0){ //service_warranty
                if($new_warranty->product_warranty == 1){ //Part Warranty

                    $product_part_names         = json_decode($request->input('part_names'));
                    $product_part_duration      = json_decode($request->input('part_duration'));

                    foreach($product_part_names as $pn_key => $part_name){
                        if($part_name != ''){
                            $temp_part_warranty                     = new AppWarrantyPart();
                            $temp_part_warranty->app_warranty_id    = $new_warranty->id;
                            $temp_part_warranty->part_name          = $part_name;
                            $temp_part_warranty->part_warranty_duration    = $product_part_duration[$pn_key];
                            $temp_part_warranty->save();
                        }
                    }
                }
            }

            if($request->input('extended')){
                $extented_warranty                      = new AppExtendedWarranty();
                $extented_warranty->app_warranty_id     = $new_warranty->id;
                $extented_warranty->warranty_period     = $request->input('extended_warranty_period');
                $extented_warranty->provider_name       = $request->input('extended_provider_name');
                $extented_warranty->provider_phone      = $request->input('extended_provider_phone');
                $extented_warranty->save();
            }
        }
        $success                                        = ['status' => true];
        $success['message']                             = 'Warranty Added!';
        echo json_encode($success);
        return ;
    }

    public function warrantyList(Request $request){
        $warranty_lists                                 = AppProductWarranty::where(['user_id' => Auth::user()->id])->orderBy('id','desc')->get();
        $success                                        = ['status' => true];
        if($warranty_lists){
            foreach($warranty_lists as $warranty_list){
                $temp_data                              = [
                                                            'warranty_id'           => $warranty_list->id,
                                                            'product_name'          => $warranty_list->product_name,
                                                            'code'                  => $warranty_list->scan_id,
                                                            'brand_name'            => $warranty_list->product_brand,
                                                            'warranty_type'         => $warranty_list->type,
                                                            'product_warranty'      => $warranty_list->product_warranty,
                                                            'waranty_period'        => (int)$warranty_list->waranty_period,
                                                            'purchase_point_name'   => $warranty_list->purchase_point_name,
                                                            'purchase_point_phone'  => $warranty_list->purchase_point_phone,
                                                            'service_center_name'   => $warranty_list->service_center_name,
                                                            'service_center_phone'  => $warranty_list->service_center_phone,
                                                            'invoice_date'          => date('d-M-Y',strtotime($warranty_list->invoice_date)),
                                                            'invoice_file'          => $warranty_list->invoice_file,
                                                            'service_invoice_file'  => $warranty_list->service_invoice_file,
                                                            'note'                  => $warranty_list->note,
                                                            'registration_type'     => $warranty_list->registration_type,
                                                            'date'                  => date('d-M-Y', strtotime($warranty_list->created_at)),
                                                        ];
                if($warranty_list->product_warranty){
                    $temp_data['product_warranty']      = false;
                }else{
                    $temp_data['product_warranty']      = true;
                }
                if($warranty_list->product_id != ''){
                    $temp_data['product_image']         = 'storage/app/images/'.$warranty_list->product->product_image;
                }else{
                    $temp_data['product_image']         = 'storage/app/warranty_files/'.$warranty_list->product_image;
                }

                if($warranty_list->invoice_file){
                    $temp_data['invoice_file']          = 'storage/app/warranty_files/'.$warranty_list->invoice_file;
                }

                if($warranty_list->service_invoice_file){
                    $temp_data['service_invoice_file']  = 'storage/app/warranty_files/'.$warranty_list->service_invoice_file;
                }

                if(sizeof($warranty_list->partWarranty)){
                    $part_names                         = [];
                    $part_duration                      = [];
                    foreach ($warranty_list->partWarranty as $parts) {
                        $part_names[]                   = $parts->part_name;
                        $part_duration[]                = (int)$parts->part_warranty_duration;
                    }
                    $temp_data['part_name']             = $part_names;
                    $temp_data['part_duration']         = $part_duration;
                }
                $temp_data['extended_warranty_status']  = false;

                if($warranty_list->extedndedWarranty){
                    $temp_data['extended_warranty_status']  = true;
                    $temp_data['extended_warranty']     = [
                                                        'warranty_period'   => (int)$warranty_list->extedndedWarranty->warranty_period,
                                                        'provider_name'     => $warranty_list->extedndedWarranty->provider_name,
                                                        'provider_phone'    => $warranty_list->extedndedWarranty->provider_phone
                                                        ];
                }
                $success['warranty_list'][]             = $temp_data;
            }
        }        
        $success['message']                             = 'Warranty List!';
        echo json_encode($success);
        return ;
    }

    public function warrantyDetails(Request $request){

        $warranty_details                               = AppProductWarranty::where(['user_id' => Auth::user()->id])->find($request->input('warranty_id'));
        $success                                        = ['status' => true];
        if($warranty_details){
            $temp_data      = [
                                'product_name'          => $warranty_details->product_name,
                                'code'                  => $warranty_details->scan_id,
                                'brand_name'            => $warranty_details->product_brand,
                                'warranty_type'         => $warranty_details->type,
                                'product_warranty'      => $warranty_details->product_warranty,
                                'waranty_period'        => (int)$warranty_details->waranty_period,
                                'purchase_point_name'   => $warranty_details->purchase_point_name,
                                'purchase_point_phone'  => $warranty_details->purchase_point_phone,
                                'service_center_name'   => $warranty_details->service_center_name,
                                'service_center_phone'  => $warranty_details->service_center_phone,
                                'invoice_date'          => date('d-M-Y', strtotime($warranty_details->invoice_date)),
                                'invoice_file'          => $warranty_details->invoice_file,
                                'service_invoice_file'  => $warranty_details->service_invoice_file,
                                'note'                  => $warranty_details->note,
                                'registration_type'     => $warranty_details->registration_type,
                                'date'                  => date('d-M-Y', strtotime($warranty_details->created_at)),
                            ];
            if($warranty_details->product_warranty){
                $temp_data['product_warranty']      = false;
            }else{
                $temp_data['product_warranty']      = true;
            }
            if($warranty_details->product_id != ''){
                $temp_data['product_image']             = 'storage/app/images/'.$warranty_details->product->product_image;
            }else{
                $temp_data['product_image']             = 'storage/app/warranty_files/'.$warranty_details->product_image;
            }

            if(sizeof($warranty_details->partWarranty) > 0){
                $part_names                             = [];
                $part_duration                          = [];
                foreach ($warranty_details->partWarranty as $parts) {
                    $part_names[]                       = $parts->part_name;
                    $part_duration[]                    = (int)$parts->part_warranty_duration;
                }
                $temp_data['part_name']                 = $part_names;
                $temp_data['part_duration']             = $part_duration;
            }
            $temp_data['extended_warranty_status']      = false;
            if($warranty_details->extedndedWarranty){
                $temp_data['extended_warranty_status']  = true;
                $temp_data['extended_warranty'] = [
                                                'warranty_period'   => (int)$warranty_details->extedndedWarranty->warranty_period,
                                                'provider_name'     => $warranty_details->extedndedWarranty->provider_name,
                                                'provider_phone'    => $warranty_details->extedndedWarranty->provider_phone
                                            ];
            }
            $success['warranty_details'][]              = $temp_data;
        }
        $success['message']                             = 'Warranty Details!';
        echo json_encode($success);
        return ;
    }

    public function warrantyUpdate(Request $request){
        $validator = Validator::make($request->all(), [
            'product_name'          => 'required',
            'product_brand'         => 'required',
            'service'               => 'required',
            'whole_product'         => 'required',
            'waranty_period'        => 'required',
            'purchase_point_name'   => 'required',
            'purchase_point_phone'  => 'required',
            'service_center_name'   => 'required',
            'service_center_phone'  => 'required',
            'invoice_date'          => 'required',
        ]);
        
        if($validator->fails()){
            return $this->sendError('Validation Error.', $validator->errors());       
        }        
        
        Log::info(print_r($request->all(),true));
        if($request->input('invoice_file')){
            $validator = Validator::make($request->all(), [
                'invoice_file'         => 'required|mimes:jpeg,jpg,png,gif|max:10000'
            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        if($request->input('service_invoice_file')){
            $validator = Validator::make($request->all(), [
                'service_invoice_file'         => 'required|mimes:jpeg,jpg,png,gif|max:10000'
            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        if($request->input('product_image')){
            $validator = Validator::make($request->all(), [
                'product_image'         => 'required|mimes:jpeg,jpg,png,gif|max:10000'
            ]);
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }

        if($request->input('extended ') == 1){
            $validator = Validator::make($request->all(), [
                'app_warranty_id'       => 'required',
                'warranty_period'       => 'required',
                'provider_name'         => 'required',
                'provider_phone'        => 'required',
            ]);       
            if($validator->fails()){
                return $this->sendError('Validation Error.', $validator->errors());       
            }
        }
        $warranty                                                   = AppProductWarranty::where(['user_id' => Auth::user()->id])
                                                                        ->find($request->input('warranty_id'));
        if($warranty){
            if($warranty->scan_id != ''){
                $deooded_code                                       = explode('-', $warranty->scan_id );
                $productDetails                                     = Product::find($deooded_code[2]);            
            }
            if(isset($productDetails)){
                $warranty->product_id                               = $productDetails->id;
                $warranty->product_name                             = $productDetails->product_name;
                $warranty->product_brand                            = $productDetails->brand_name;
            }else{
                $warranty->product_name                             = $request->input('product_name');
                $warranty->product_brand                            = $request->input('product_brand');
                if($request->hasFile('product_image')){
                    $product_image_file             = time().Auth::user()->id.'PI.'.$request->product_image->extension();
                    if($request->product_image->storeAs('public/app/warranty_files', $product_image_file)){
                        $warranty->product_image = $product_image_file;
                    }
                }
            }
            $warranty->type                                         = $request->input('service');
            $warranty->product_warranty                             = $request->input('whole_product');
            $warranty->waranty_period                               = $request->input('waranty_period');
            $warranty->purchase_point_name                          = $request->input('purchase_point_name');
            $warranty->purchase_point_phone                         = $request->input('purchase_point_phone');
            $warranty->service_center_name                          = $request->input('service_center_name');
            $warranty->service_center_phone                         = $request->input('service_center_phone');
            $warranty->invoice_date                                 = date('Y-m-d H:i:s', strtotime($request->input('invoice_date')));
            $warranty->note                                         = $request->input('note');
            $warranty->registration_type                            = 0;

            if($request->hasFile('invoice_file')){
                $product_invoice_file               = time().Auth::user()->id.'IF.'.$request->invoice_file->extension();
                if($request->invoice_file->storeAs('public/app/warranty_files', $product_invoice_file)){
                    $warranty->invoice_file     = $product_invoice_file;
                }
            }

            if($request->hasFile('service_invoice_file')){
                $product_service_invoice_file       = time().Auth::user()->id.'SIF.'.$request->service_invoice_file->extension();
                if($request->service_invoice_file->storeAs('public/app/warranty_files', $product_service_invoice_file)){
                    $warranty->service_invoice_file = $product_service_invoice_file;
                }
            }
            
            if($warranty->type == 0){ //service_warranty
                if($warranty->product_warranty == 0){ //whole_product_warranty
                    $warranty->waranty_period                       = $request->input('waranty_period');
                }
            }else{ // replacement
                $warranty->waranty_period                           = $request->input('waranty_period');
            }

            if($warranty->save()){
                AppWarrantyPart::where('app_warranty_id', $warranty->id)->delete();
                AppExtendedWarranty::where('app_warranty_id', $warranty->id)->delete();
                if($warranty->type == 0){ //service_warranty
                    if($warranty->product_warranty == 1){ //Part Warranty
                        $product_part_names                             = json_decode($request->input('part_names'));
                        $product_part_duration                          = json_decode($request->input('part_duration'));
                        foreach($product_part_names as $pn_key => $part_name){
                            if($part_name != ''){
                                $temp_part_warranty                     = new AppWarrantyPart();
                                $temp_part_warranty->app_warranty_id    = $warranty->id;
                                $temp_part_warranty->part_name          = $part_name;
                                $temp_part_warranty->part_warranty_duration = $product_part_duration[$pn_key];
                                $temp_part_warranty->save();
                            }
                        }
                    }
                }

                if($request->input('extended')){
                    $extented_warranty                                  = new AppExtendedWarranty();
                    $extented_warranty->app_warranty_id                 = $warranty->id;
                    $extented_warranty->warranty_period                 = $request->input('extended_warranty_period');
                    $extented_warranty->provider_name                   = $request->input('extended_provider_name');
                    $extented_warranty->provider_phone                  = $request->input('extended_provider_phone');
                    $extented_warranty->save();
                }
            }
        }
        $success                                                        = ['status' => true];
        $success['message']                                             = 'Warranty Updated Successfully!';
        echo json_encode($success);
        return ;
    }
}