<?php
    
namespace App\Http\Controllers;
    
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Categories;
use App\Models\Company;
use Spatie\Permission\Models\Role;
use DB;
use Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
use Session;
use Redirect;
use Illuminate\Support\Collection;
use DataTables;

    
class CatergoriesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
         $this->middleware('permission:category-list|category-create|category-edit|category-delete', ['only' => ['index','store']]);
         $this->middleware('permission:category-create', ['only' => ['create','store']]);
         $this->middleware('permission:category-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:category-delete', ['only' => ['destroy']]);
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $session_company    = Session::get('company');
        $data               = Categories::orderBy('id','ASC')->where('company_id',$session_company)->paginate(10);
        $companies          = Company::pluck('company_name','id')->all();

        if($request['company_id']!=null) {
            $company_id=$request['company_id'];
        }
        elseif($request['company_id']==null && $session_company==null) {
                $company_id=1;

            }
        elseif($session_company!=null){
            $company_id=$session_company;
        }
        else {
            $company_id=$request['company_id'];
        }

        return view('categories.index', compact('data','companies','company_id'));
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(Request $request)
    {
        try {
            // $categories = Categories::get();
            // return view('categories.create',compact('categories'));
            $session_company=Session::get('company');

            if($request['company_id']!=null) {
                $company_id=$request['company_id'];
            }
            elseif($request['company_id']==null && $session_company==null) {
                $company_id=1;
            }
            elseif($session_company!=null){
                $company_id=$session_company;
            }
            else {
                $company_id=$request['company_id'];
            }
            
            $categories = Categories::where('parent_id',0)->where('company_id',$company_id)->get();  
            
            if($company_id==1){
                $companies = Company::pluck('company_name','id')->all();
            }
            else {
                $companies = Company::where('id',$company_id)->pluck('company_name','id')->all();
            }

            return view('categories.create',compact('categories','companies'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController create :- " . $e->getMessage());
        } 
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // try {
            $this->validate($request, [
                'category' => 'required',
            ]);
        
            // $categories = Categories::create(['category' => $request->input('category')]);

            $categories = new Categories;
            $categories->parent_id      = $request->input('parent_id');
            $categories->category       = $request->input('category');
            $categories->company_id     = $request->input('companies');

            $categories->save();

            return redirect()->route('categories.index',['company_id' => $categories->company_id])
                            ->with('success','Category created successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController store :- " . $e->getMessage());
        // } 
    
    }
    
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $category = Categories::find($id);
            return view('categories.show',compact('category'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController show :- " . $e->getMessage());
        } 
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, $id)
    {
        // try {
            $category_info = Categories::find($id);

            $session_company=Session::get('company');
            $session_company=Session::get('company');

            if($request['company_id']!=null) {
                $company_id=$request['company_id'];
            }
            elseif($request['company_id']==null && $session_company==null) {
                $company_id=1;
            }
            elseif($session_company!=null){
                $company_id=$session_company;
            }
            else {
                $company_id=$request['company_id'];
            }
            
            $categories = Categories::where('parent_id',0)->where('company_id',$company_id)->where('id','!=',$id)->get();  
            
            if($company_id==1){
                $companies = Company::pluck('company_name','id')->all();
            }
            else {
                $companies = Company::where('id',$company_id)->pluck('company_name','id')->all();
            }


            if($company_id==1) {
                $companies = Company::pluck('company_name','id')->all();
            }
            else {
                $companies = Company::where('id',$company_id)->pluck('company_name','id')->all();
            }

            return view('categories.edit',compact('category_info','categories','companies'));
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController edit :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // try {
            $this->validate($request, [
                'category' => 'required',
            ]);
            
            $categories = Categories::find($id);
            
            if($categories->parent_id==0 && $request->input('parent_id')!=0) {
                $sub_categories = Categories::where('parent_id',$id)->get();
                if(count($sub_categories)>0) {
                    return redirect()->route('categories.edit',$id)
                            ->with('error','Sorry, this category has subcategories. If you want to change the parent, you must first change the parent of its subcategories and try again.');
                }
            }

            if($categories->company_id != $request->input('companies')) {
                $sub_categories = Categories::where('parent_id',$id)->get();
                if(count($sub_categories)>0) {
                    return redirect()->route('categories.edit',$id)
                            ->with('company_error','Sorry, sub-categories are currently assigned to company. Please unassign the sub-categories before making changes.');
                }
            }

            $categories->category      = $request->input('category');
            $categories->company_id    = $request->input('companies');
            // $categories->parent_id  = 0;
            $categories->parent_id      = $request->input('parent_id');



            $categories->save();

            return redirect()->route('categories.index',['company_id' => $categories->company_id])
                            ->with('success','Category updated successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController update :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {

            $categories = Categories::find($id);

            if($categories->parent_id==0) {
                $sub_categories = Categories::where('parent_id',$id)->get();
                if(count($sub_categories)>0) {
                    DB::table("categories")->where('parent_id',$id)->delete();
                }
            }

            DB::table("categories")->where('id',$id)->delete();
            DB::table("categories")->where('parent_id',$id)->delete();
            return redirect()->route('categories.index',['company_id' => $categories->company_id])
                        ->with('success','Category deleted successfully');
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController destroy :- " . $e->getMessage());
        } 
    }

    public function categoryList(Request $request) {
        try {
            $Categories         = new Categories;

            if($request['company_id']==null)   {
                $company        = 0;
                }
            else {
                $company        = $request['company_id'];
            }
            $company_id         = $company;
                
            $result             = Categories::with('subCategory')->where(['company_id' => $company_id, 'parent_id' => 0 ])->get();
            $category   = collect($result);
            $datatables         = datatables()->of($category)->addColumn('sl_no', function () {
                return '';
            });

            $datatables         = $datatables->addColumn('category', function ($category) {
                                        if($category->subCategory->isEmpty()){
                                            return '<strong>'.$category->category.'</strong>';
                                        }else{
                                            
                                            return '<strong>'.$category->category.'</strong>&nbsp;&nbsp;<span class="dt-control"></span>';
                                        }
                                });
            $datatables         = $datatables->addColumn('no_1', function ($category) {
                return '';            
            });

            $datatables         = $datatables->addColumn('sub_category', function ($category) {
                return '';            
            });


            
            $datatables         = $datatables->addColumn('data1', function ($category) {
                    if($category->subCategory->isEmpty()){
                        return '[]';
                    }else{
                        $temp_data              = [];
                        $user                   = Auth()->user();
                        if($category->subCategory){
                            foreach($category->subCategory as $subcategory){
                                $temp_html          = '<div class="action-icons">';
                                    if($user->can('category-edit')){
                                            // $temp_html.='<a href="' . url("/sub-category/edit/".$subcategory->id) . '" title="Edit" ><i class="fas fa-edit"></i></a>';
                                            $temp_html.='<a href="' . route("categories.edit", $subcategory->id). '" title="Edit" ><i class="fas fa-edit"></i></a>';
                                        }
                                        if($user->can('category-delete')){

                                            if($subcategory->subCategoryProducts->isEmpty()){
                                                $confirmation_msg="Are you sure you want to delete this category ".$subcategory->category." ?";
                                            }
                                            else {
                                                if($subcategory->subCategoryProducts->isNotEmpty()){
                                                    $confirmation_msg="Sorry, this category has some products. Do you want to proceed?";
                                                    $confirmation_msg .= '<br/> <br/>';
                                                    $confirmation_msg .= '<h5>Product List</h5>';
                                                    foreach($subcategory->subCategoryProducts as $t_key => $t_products){

                                                        $confirmation_msg .= '<a target="_blank" href="' . route("products.show", $t_products->id) . '" title="Show" class="De_view">'.(++$t_key).'. '. $t_products->product_name.'</a><br/>';
                                                    }
                                                }else{
                                                    $confirmation_msg="Are you sure you want to delete this category ".$subcategory->category." ?";
                                                }
                                                
                                            }



                                            $temp_html.='<button type="button" class="btn btn-xs btn-flat show_confirm delete-btn" data-bs-toggle="modal" data-bs-target="#deleteModal'. $subcategory->id .'" data-category-id="'. $subcategory->id .'" title="Delete"><i class="fa fa-trash-o" aria-hidden="true"> </i></button>';
                                            $temp_html.='<div class="modal fade" id="deleteModal'. $subcategory->id .'" tabindex="-1" aria-labelledby="deleteModalLabel" aria-hidden="true">
                                            <div class="modal-dialog">
                                            <div class="modal-content">
                                            <div class="modal-header">
                                                <h5 class="modal-title" id="deleteModalLabel">Delete Confirmation</h5>
                                                </div>
                                                <div class="modal-body">'.$confirmation_msg.
                                                '</div>
                                                <div class="modal-footer">
                                                <button type="button" class="btn btn-secondary modal-btn" data-dismiss="modal">Cancel</button>';
                                                    if($subcategory->subCategoryProducts->isEmpty()){
                                                        $temp_html.= '<form method="POST" action="'.route("categories.destroy", $subcategory->id).'">
                                                        <input name="_method" type="hidden" value="DELETE">
                                                        <input type="hidden" name="_token" value="'.csrf_token().'" />
                                                            <button type="submit" class="btn btn-danger text-right modal-btn">Delete</button>
                                                        </form>';
                                                    }
                                                $temp_html.= '</div>
                                            </div>
                                            </div>
                                            </div>';
                                        }
                                $temp_html    .= '</div>';
                                $temp_data[]        = ['category' => $subcategory->category,
                                                        'id'        => $subcategory->id,
                                                        'actions'   => $temp_html
                                                    ];
                            }
                            return $temp_data;
                        }else{
                            return [];
                        }
                    }
                
            });



            $datatables = $datatables->addColumn('actions', function ($category) {
                $user               = Auth()->user();
                $actions            ='<div class="action-icons">';
                /*
                $actions            .= '<a href="' . route("categories.show", $category->id) . '" title="Show" ><i class="fas fa-eye"></i></a>&nbsp;';
                */
                if($user->can('category-edit')){
                    $actions.='<a href="' . route("categories.edit", $category->id) . '" title="Edit" ><i class="fas fa-edit"></i></a>';
                }
                /*
                if($user->can('category-create')){
                    $actions.='<a href="'.url('/sub-category/create/'.$category->id).'" title="Add Sub Category" ><i class="fas fa-plus"></i></a>&nbsp;';
                }

                */
                if($user->can('category-delete')){
                //     $actions.='<form method="POST" action="'.route("categories.destroy", $category->id).'">
                //     <input name="_method" type="hidden" value="DELETE">
                //     <input type="hidden" name="_token" value="'.csrf_token().'" />
                //     <button onclick="return confirm(\''."Are you sure to delete this category ?".'\')" type="submit" class="btn btn-xs btn-flat show_confirm" data-toggle="tooltip" title="Delete"> <i class="fa fa-trash-o" aria-hidden="true"> </i></button>
                // </form>';
                    if($category->subCategory->isEmpty() && $category->products->isEmpty()){
                        $confirmation_msg="Are you sure you want to delete this category ".$category->category." ?";
                    }
                    else {
                        if($category->subCategory->isNotEmpty()){
                            $confirmation_msg="Sorry, this category has subcategories / Products.";
                            if($category->products->isNotEmpty()){
                                ///$confirmation_msg="Sorry, this category has some products. Do you want to proceed?";

                                $confirmation_msg .= '<br/> <br/>';
                                $confirmation_msg .= '<h5>Product List</h5>';
                                foreach($category->products as $t_key => $t_products){
                                    $confirmation_msg .= '<a target="_blank" href="' . route("products.show", $t_products->id) . '" title="Show" class="De_view">'.(++$t_key).'. '. $t_products->product_name.'</a><br/>';
                                }
                            }
                        }elseif($category->products->isNotEmpty()){
                            $confirmation_msg="Sorry, this category has some products. Do you want to proceed?";
                            foreach($category->products as $t_key => $t_products){
                                $confirmation_msg .= '<br/> '.(++$t_key).'. '. $t_products->product_name;
                            }
                        }else{
                            $confirmation_msg="Are you sure you want to delete this category".$category->category." ?";
                        }
                        
                    }

                    $actions.='<button type="button" class="btn btn-xs btn-flat show_confirm delete-btn" data-bs-toggle="modal" data-bs-target="#deleteModal'. $category->id .'" data-category-id="'. $category->id .'"  title="Delete"><i class="fa fa-trash-o" aria-hidden="true"> </i></button>';
                    
                    
                    $actions.='<div class="modal fade" id="deleteModal'. $category->id .'" tabindex="-1" aria-labelledby="deleteModalLabel" aria-hidden="true">
                    <div class="modal-dialog">
                    <div class="modal-content">
                    <div class="modal-header">
                        <h5 class="modal-title" id="deleteModalLabel">Delete Confirmation</h5>
                        </div>
                        <div class="modal-body">
                        '.$confirmation_msg.'
                        </div>
                        <div class="modal-footer">
                        <button type="button" class="btn btn-secondary modal-btn" data-dismiss="modal">Cancel</button>';
                                if($category->subCategory->isEmpty() && $category->products->isEmpty()){
                                    $actions.= '<form method="POST" action="'.route("categories.destroy", $category->id).'">
                                    <input name="_method" type="hidden" value="DELETE">
                                    <input type="hidden" name="_token" value="'.csrf_token().'" />
                                        <button type="submit" class="btn btn-danger text-right modal-btn">Delete</button>
                                    </form>';
                                }
                        $actions.= '</div>
                    </div>
                    </div>
                    </div>';
                }
                $actions.='</div>';
                return $actions;


            });

            $datatables = $datatables->escapeColumns([])->toJson();
                return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController categoryList :- " . $e->getMessage());
        } 


    }


    public function companyCategoryList(Request $request) {
        // used to update the content of div with id parent_category_html in category edit/add pages based on company selected
        $categories = Categories::with('subCategory')->where(['company_id' => $request['company_id'], 'parent_id' => 0 ])->get();


            $html ='<strong><label>Parent Category:</label></strong>
                        <select name="parent_id" class = "form-control form-control-sm">
                            <option value=0>No parent, This is a main category</option>';
                        foreach($categories as $category) {
                            $html .='<option value="'.$category->id.'">'.$category->category.'</option>';
                        }
                        $html .='</select>';
        echo $html;
    }


    public function companyMainCategoryList(Request $request) {
        // used to update the content of div with id parent_category_html in category edit/add pages based on company selected
        $categories = Categories::with('subCategory')->where(['company_id' => $request['company_id'], 'parent_id' => 0 ])->get();


            $html ='<label>Main Category:<span class="form-compulsory"> * </span></label>';
                if(count($categories)>0) {
                    $html .='<select id="category_id" name="category" class="form-control form-control-sm">';
                    $html .='<option value=0>-- Main category not selected --</option>';  
                    foreach($categories as $category) {
                        $html .='<option value="'.$category->id.'">'.$category->category.'</option>';
                    }
                }
                else {
                    $html .='<br/><label> No categories added for this company. You can add categories if required. </label>';
                }
                    $html .='</select>';
        echo $html;
    }
    

    public function companySubcategoryList(Request $request) {
        // used to update the content of div with id subcategory_html in products edit/add pages based on main category selected
        $categories = Categories::where(['parent_id' => $request['parent_id'] ])->get();
        $html ='<div class="col-xs-10 col-sm-10 col-md-10"><div class="form-group" >';
        $html .='<strong><label>Subcategory :</label></strong>';
        if($request['parent_id']==0||$request['parent_id']==null) {
            $html .='<br/><label></label>';
        }
        else {
            
                $html .='<select id="subcategory_id" name="subcategory" class = "form-control form-control-sm">'; 
                $html .='<option value="0">-- Sub category not selected --</option>';
            if(count($categories)>0) {
                foreach($categories as $category) {
                    $html .='<option value="'.$category->id.'">'.$category->category.'</option>';
                }
            }
            // else {
            //     $html .='<br/><label> No subcategories for this category. You can add subcategories if required. </label>';
            // }
                $html .='</select>';
        }
        $html .='</div></div>';
        $html .='<div class="col-xs-2 col-sm-2 col-md-2  add-cat-btn">
        <button type="button" class="btn btn-xs btn-flat sub-cat-btn" data-bs-toggle="modal" data-bs-target="#addCatModal"><i class="fa fa-plus" aria-hidden="true"></i></button>
      </div>';

        echo $html;
    }


    public function addCategoryModal(Request $request)
    {
        // try {
            $categories = new Categories;

            // $this->validate($request, [
            //     'new_category' => 'required',
            // ]);

            if($request['parent_id']!="" && $request['new_cateory']!="") {
                $categories->parent_id  = $request['parent_id'];
                $categories->category   = $request['new_cateory'];
                $categories->company_id = $request['company_id'];
                $categories->save();
            }

            $html ='<option value="'.$categories->id.'" selected>'.$categories->category.'</option>';

            echo $html;

            // return Redirect::back()->with('success','Category created successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController store :- " . $e->getMessage());
        // } 
    
    }

    public function validateNewCategory(Request $request)
    {
        $category_id                    = $request['category_id'];
        if($category_id=="") {
            $category_check                 = Categories::where(['category' => $request['new_category'], 'company_id' => $request['company_id'] ])->first();
        }
        else {
            $category_check                 = Categories::where(['category' => $request['new_category'], 'company_id' => $request['company_id'] ])->where('id','!=',$category_id)->first();
        }
        $response                       = [];
        if($category_check){
            $response['status']         = false;
            $response['msg']            = 'This category already exists';

        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }



    
   


// Commented all functions for subCatgeory - parent class field added in category add/edit for manage category/subcategory 


//    public function subCategoryCreate($id)
//     {
//         try {
//             $category       = Categories::find($id);
//             if(!$category){
//                 return Redirect::back();
//             }
//             return view('categories.sub-create',compact('category'));
//         }
//         catch (\Exception $e) {
//             Log::error("Error in CategoriesController create :- " . $e->getMessage());
//         } 
//     }
    
//     /**
//      * Store a newly created resource in storage.
//      *
//      * @param  \Illuminate\Http\Request  $request
//      * @return \Illuminate\Http\Response
//      */
//     public function subCategoryStore(Request $request)
//     {
//         // try {
//             $this->validate($request, [
//                 'parent_category'   => 'required',
//                 'category' => 'required',
//             ]);
//             //dd($request->all());
//             // $categories = Categories::create(['category' => $request->input('category')]);
            
//             $request->input('category');

//             $categories = new Categories;
//             $categories->category   = $request->input('category');
//             $categories->company_id = Session::get('company');
//             $categories->parent_id  = $request->input('parent_category');;
//             $categories->save();

//             return redirect()->route('categories.index')
//                             ->with('success','Category created successfully');
//         // }
//         // catch (\Exception $e) {
//         //     Log::error("Error in CategoriesController store :- " . $e->getMessage());
//         // } 
    
//     }
    
//     /**
//      * Display the specified resource.
//      *
//      * @param  int  $id
//      * @return \Illuminate\Http\Response
//      */
//     public function subCategoryShow($id)
//     {
//         try {
//             $category = Categories::find($id);
//             return view('categories.show',compact('category'));
//         }
//         catch (\Exception $e) {
//             Log::error("Error in CategoriesController show :- " . $e->getMessage());
//         } 
//     }
    
//     /**
//      * Show the form for editing the specified resource.
//      *
//      * @param  int  $id
//      * @return \Illuminate\Http\Response
//      */
//     public function subCategoryEdit($id)
//     {
//         try {
//             $category = Categories::with('parentCategory')->find($id);
//             return view('categories.sub-edit',compact('category'));
//         }
//         catch (\Exception $e) {
//             Log::error("Error in CategoriesController edit :- " . $e->getMessage());
//         } 
//     }
    
//     /**
//      * Update the specified resource in storage.
//      *
//      * @param  \Illuminate\Http\Request  $request
//      * @param  int  $id
//      * @return \Illuminate\Http\Response
//      */
//     public function subCategoryUpdate(Request $request, $id)
//     {
//         // try {
//             $this->validate($request, [
//                 'category' => 'required',
//             ]);
        
//             $categories = Categories::find($id);
//             $categories->category = $request->input('category');
//             $categories->company_id = Session::get('company');
//             $categories->save();

//             return redirect()->route('categories.index')
//                             ->with('success','Category updated successfully');
//         // }
//         // catch (\Exception $e) {
//         //     Log::error("Error in CategoriesController update :- " . $e->getMessage());
//         // } 
//     }
    
//     /**
//      * Remove the specified resource from storage.
//      *
//      * @param  int  $id
//      * @return \Illuminate\Http\Response
//      */
//     public function subCategoryDestroy($id)
//     {
//         try {
//             DB::table("categories")->where('id',$id)->delete();
//                 return redirect()->route('categories.index')
//                         ->with('success','Category deleted successfully');
//         }
//         catch (\Exception $e) {
//             Log::error("Error in CategoriesController destroy :- " . $e->getMessage());
//         } 
//     }


}