<?php
    
namespace App\Http\Controllers;
    
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Company;
use App\Models\Categories;
use App\Models\CompanyCategory;
use App\Models\CompanyUser;
use App\Models\ContactPerson;
use App\Models\Product;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Traits\HasRoles;
use DB;
use Hash;
use Session;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
    
class CompanyController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
         $this->middleware('permission:company-list|company-create|company-edit|company-delete', ['only' => ['index','store']]);
         $this->middleware('permission:company-create', ['only' => ['create','store']]);
         $this->middleware('permission:company-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:company-delete', ['only' => ['destroy']]);
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        try {            
            return view('companies.index');
        }
        catch (\Exception $e) {
            Log::error("Error in CompanyController index :- " . $e->getMessage());
        } 
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        // try {
            $categories = Categories::pluck('category','category')->all();
            // $countries = DB::table('utility_country')->select('name')->all();

            // $countries = DB::table('country')
            //     ->select('nicename')
            //     ->orderBy('nicename', 'ASC')->get();

            $countries = DB::table('countries')->pluck('name','id')->all();
            $states = DB::table('states')->where('country_id',101)->orderBy('name')->pluck('name','id')->all();

            // dd($countries);

            return view('companies.create',compact('categories','countries','states'));
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CompanyController create :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // try {
            $this->validate($request, [
                'company_name'=> 'required',
                'company_code'=> 'required|unique:company,company_code|min:2|max:2',
                'company_logo'=> 'mimes:jpeg,jpg,png,gif|max:10000',
                'building_name'=> 'required',
                // 'area'=> 'required',
                // 'landmark'=> 'required',
                // 'town'=> 'required',
                'pin'=> 'integer|digits_between:6,6',
                // 'state'=> 'required',
                // 'country'=> 'required',
                'email'=> 'required|email|unique:company,email',
                // 'website'=> 'required',
                'telephone_code'=> 'required|integer|digits_between:2,3',
                'telephone'=> 'required|integer|digits_between:7,10',
                // 'note'=> 'required',
                // 'categories'=> 'required',
            ]);
            // return "hi";
            // $input = $request->all();
            
            // $company = Company::create($input);

            $company                        = new Company;
            $company->company_name          = $request->input('company_name');
            $company->company_type          = $request->input('company_type');
            $company->company_code          = $request->input('company_code');
            $company->gst_number            = $request->input('gst_number');
            
            $company_logo_name              = time().'Company.'.$request->company_logo->extension();
            if($request->company_logo->storeAs('public/app/companies', $company_logo_name)){
                $company->company_logo      = $company_logo_name; 
            }

            $company->telephone_code        = $request->input('telephone_code');
            $company->telephone             = $request->input('telephone');
            $company->email                 = $request->input('email');
            $company->website               = $request->input('website');
            $company->building_name         = $request->input('building_name');
            $company->area                  = $request->input('area');
            $company->landmark              = $request->input('landmark');
            $company->town                  = $request->input('town');
            $company->country               = $request->input('country');
            $company->state                 = $request->input('state');
            $company->pin                   = $request->input('pin');

            $company->save();

            $contact_names                  = $request->input('contact_name');
            $contact_designations           = $request->input('contact_designation');
            $contact_emails                 = $request->input('contact_email');
            $contact_cont_mob_code1s        = $request->input('contact_mob_code1');
            $contact_cont_mobile1s          = $request->input('contact_mobile1');
            $contact_cont_mob_code2s        = $request->input('contact_mob_code2');
            $contact_cont_mobile2s          = $request->input('contact_mobile2');
            
            if($contact_names)  {
                foreach ($contact_names as $key => $contact_name) {
                    $new_contact_person                 = new ContactPerson;
                    $new_contact_person->company_id     = $company->id;
                    $new_contact_person->name           = $contact_name;
                    $new_contact_person->designation    = $contact_designations[$key];
                    $new_contact_person->email          = $contact_emails[$key];
                    $new_contact_person->mob_code1      = $contact_cont_mob_code1s[$key];
                    $new_contact_person->mobile1        = $contact_cont_mobile1s[$key];
                    $new_contact_person->mob_code2      = $contact_cont_mob_code2s[$key];
                    $new_contact_person->mobile2        = $contact_cont_mobile2s[$key];
                    
                    $new_contact_person->save();
                }
            }



            // $categories = new Categories;

            // foreach($request->categories as $item => $value) {
            //     // $category = $categories->getCategoryIdByName($value);
            //     $data[$value]=array(
            //         'category'=>$value,
            //         'company_id'=>$company->id,
            //         'parent_id'=>0,
                    
            //     );
            // }
            // Categories::insert($data);

            // $categories = $request->input('categories');
            // if($categories) {
            //     foreach ($categories as $category) {
            //         $comp_category = new CompanyCategory;
            //         $comp_category["company_id"] = $company->id;
            //         $comp_category["category_id"] = $category;
            //         CompanyCategory::insert($data);
            //     }

            // }
            
            // return $request->input('categories');
            // $comp_category->company_id = $company->id;
            // $comp_category->category_id = $request->input('categories');
            // $comp_category->save();

            // $comp_category = new CompanyCategory;
            // $comp_category["company_id"] = $company->id;
            // $comp_category["category_id"] = $request->input('categories');
            // CompanyCategory::insert($comp_category);

        
            return redirect()->route('companies.index')
                            ->with('success','Company details added successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CompanyController store :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        // try {
            $company = Company::find($id);
            $country = DB::table('countries')->where('id',$company->country)->select('name')->first();
            $state = DB::table('states')->where('id',$company->state)->select('name')->first();
            $contact_persons = DB::table('contact_persons')->where('company_id',$id)->get();
            return view('companies.show',compact('company','country','state','contact_persons'));
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CompanyController show :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        try {

            $company = Company::find($id);
            $categories = Categories::where('company_id',$id)->pluck('category','id')->all();

            // $companyCategory = $company->companyCategories->categoryName->pluck('category','category')->all();
            // $companyCategory = $company->companyCategories->pluck('category_id')->all();
            $companyCategory = Categories::where('company_id',$id)->pluck('id')->all();

            $countries = DB::table('countries')->pluck('name','id')->all();
            $states = DB::table('states')->where('country_id',$company->country)->orderBy('name')->pluck('name','id')->all();

            $contact_persons = DB::table('contact_persons')->where('company_id',$id)->get();

            // return $companyCategory;
        
            return view('companies.edit',compact('company','categories','companyCategory','countries','states','contact_persons'));
            // return view('companies.edit',compact('company','categories'));
            
        }
        catch (\Exception $e) {
            Log::error("Error in CompanyController edit :- " . $e->getMessage());
        } 
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // try {
            $this->validate($request, [
                'company_name'=> 'required',
                'company_code'=> 'required|min:2|max:2|unique:company,company_code,'.$id,
                'company_logo'=> 'mimes:jpeg,jpg,png,gif|max:10000',
                'building_name'=> 'required',
                // 'area'=> 'required',
                // 'landmark'=> 'required',
                // 'town'=> 'required',
                'pin'=> 'integer|digits_between:6,6',
                // 'state'=> 'required',
                // 'country'=> 'required',
                'email'=> 'required|email|unique:company,email,'.$id,
                // 'website'=> 'required',
                'telephone_code'=> 'required|integer|digits_between:2,3',
                'telephone'=> 'required|integer|digits_between:7,10',
                // 'note'=> 'required',
                // 'categories'=> 'required',
                
            ]);

            if($request->hasFile('company_logo')){
                $this->validate($request, [
                    'company_logo'          => 'mimes:jpeg,jpg,png,gif|max:10000',
                ]);
            }
        
            // $input = $request->all();
        
            $company = Company::find($id);

            $company->company_name          = $request->input('company_name');
            $company->company_type          = $request->input('company_type');
            $company->company_code          = $request->input('company_code');
            $company->gst_number            = $request->input('gst_number');

            if($request->hasFile('company_logo')){
                $company_logo_name         = time().'Company.'.$request->company_logo->extension();
                if($request->file('company_logo')->storeAs('public/app/companies',$company_logo_name)){
                    $company->company_logo = $company_logo_name;
                }
                if(Session::get('company') == $company->id) {
                    Session::forget('company_logo');
                    Session::put('company_logo',$company_logo_name);
                }
            }
            

            $company->telephone_code        = $request->input('telephone_code');
            $company->telephone             = $request->input('telephone');
            $company->email                 = $request->input('email');
            $company->website               = $request->input('website');
            $company->building_name         = $request->input('building_name');
            $company->area                  = $request->input('area');
            $company->landmark              = $request->input('landmark');
            $company->town                  = $request->input('town');
            $company->country               = $request->input('country');
            
            $states = DB::table('states')->where('country_id',$company->country)->get();
            if(count($states)>0) {
                $company->state                 = $request->input('state');
            }
            else {
                $company->state                 = 0;
            }
            
            $company->pin                   = $request->input('pin');

            $company->save();


            $contact_names                  = $request->input('contact_name');
            $contact_designations           = $request->input('contact_designation');
            $contact_emails                 = $request->input('contact_email');
            $contact_cont_mob_code1s        = $request->input('contact_mob_code1');
            $contact_cont_mobile1s          = $request->input('contact_mobile1');
            $contact_cont_mob_code2s        = $request->input('contact_mob_code2');
            $contact_cont_mobile2s          = $request->input('contact_mobile2');

            ContactPerson::where('company_id',$company->id)->delete();
            
            if($contact_names)  {
                foreach ($contact_names as $key => $contact_name) {
                    if($contact_name!=null) {
                        $new_contact_person                 = new ContactPerson;
                        $new_contact_person->company_id     = $company->id;
                        $new_contact_person->name           = $contact_name;
                        $new_contact_person->designation    = $contact_designations[$key];
                        $new_contact_person->email          = $contact_emails[$key];
                        $new_contact_person->mob_code1      = $contact_cont_mob_code1s[$key];
                        $new_contact_person->mobile1        = $contact_cont_mobile1s[$key];
                        $new_contact_person->mob_code2      = $contact_cont_mob_code2s[$key];
                        $new_contact_person->mobile2        = $contact_cont_mobile2s[$key];
                        
                        $new_contact_person->save();

                    }  
                }
            }

            // return $company;

            // $categories = new Categories;
            // $print = "";

            // DB::table('company_categories')->where('company_id',$company->id)->delete();

            // foreach($request->categories as $item => $value) {
            //     // $data1=(int)$value;
            //     // return $data1;
            //     // $print.="-".$data1;
            //     // $category = $categories->getCategoryIdByName($data1);
            //     $data[$value]=array(
            //         'company_id'=>$id,
            //         'category_id'=>(int)$value,
            //     );
            // }

            // CompanyCategory::insert($data);

            // return $data;
            // return $print;

            // $companyCategory = CompanyCategory::where(['company_id'=>$company->id])->get();
            // return $companyCategory;

            // $companyCategory->update($data);

            
            

        
            // $user->assignRole($request->input('roles'));
        
            return redirect()->route('companies.index')
                            ->with('success','Company details updated successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CompanyController update :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
          
            $model = Company::find($id);
            $model->delete();
            // CompanyUser::where('company_id',$id)->delete();   // deletes entries for all users for this company from company_users table
            // Product::where('company_id',$id)->delete();       // soft deletes all products for this company
            return redirect()->route('companies.index')
                        ->with('success','Company details deleted successfully');
        }
        catch (\Exception $e) {
            Log::error("Error in CompanyController destroy :- " . $e->getMessage());
        } 
    }

    public function companyList()
    {
        try {
            $Company = new Company;
                
            //retrieve company details
            
            $result = $Company->getDetailsCompany();
            $data = collect($result);

            
            $datatables = datatables()->of($data)->addColumn('sl_no', function () {
                return '';
            });

            $datatables =$datatables->addColumn('company_name', function ($data) {
                $company_name = $data->company_name;
                $view = '<span  style="display:none;">'.$company_name.'</span><a href="' . route("companies.show", $data->id) . '" title="Show" class="De_view">'.$company_name.'</i>';
                return $view;
            });

            // $datatables = $datatables->addColumn('contact', function ($data) {
            //     $contact="";
            //     $company=Company::find($data->id);
            //     if($data->telephone) {
            //         $contact.=$data->telephone."<br/>";
            //     }
            //     if($data->email) {
            //         $contact.=$data->email."<br/>";
            //     }
            //     return $contact;
            // });


            $datatables = $datatables->addColumn('actions', function ($data) {
                $user = Auth()->user();
                $actions='<div class="action-icons">';
                if($user->can('company-edit')){
                    $actions.='<a href="' . route("companies.edit", $data->id) . '" title="Edit" ><i class="fa fa-pencil-square-o" aria-hidden="true"></i></a>';
                }
                if($user->can('company-delete')){
                    $users = CompanyUser::where('company_id',$data->id)->get();
                    $products = Product::where('company_id',$data->id)->get();
                    if(count($products) || count($users)) {
                        $confirmation_msg="Sorry, the company $data->company_name has users or products. If you want to delete this company you have to remove related users & products first.";
                    }
                    else {                         
                        $confirmation_msg="Are you sure you want to delete the company $data->company_name ?";
                    }
                    $actions.='<button type="button" class="btn btn-xs btn-flat show_confirm delete-btn" data-bs-toggle="modal" data-bs-target="#deleteModal'. $data->id .'" data-company-id="'. $data->id .'"  title="Delete"><i class="fa fa-trash-o" aria-hidden="true"> </i></button>';
                
                    $actions.='<div class="modal fade" id="deleteModal'. $data->id .'" tabindex="-1" aria-labelledby="deleteModalLabel" aria-hidden="true">
                    <div class="modal-dialog">
                    <div class="modal-content">
                    <div class="modal-header">
                        <h5 class="modal-title" id="deleteModalLabel">Delete Confirmation</h5>
                        </div>
                        <div class="modal-body">
                        '.$confirmation_msg.'
                        </div>
                        <div class="modal-footer">
                        <button type="button" class="btn btn-secondary modal-btn" data-dismiss="modal">Cancel</button>';
                        if($products->isEmpty() && $users->isEmpty()) {
                            $actions.='<form method="POST" action="'.route("companies.destroy", $data->id).'">
                                <input name="_method" type="hidden" value="DELETE">
                                <input type="hidden" name="_token" value="'.csrf_token().'" />
                                    <button type="submit" class="btn btn-danger text-right modal-btn">Delete</button>
                                </form>';
                        }
                $actions.='</div>
                    </div>
                    </div>
                    </div>';
                }
                // $company_user = CompanyUser::where('company_id',$data->id)->get();
                // if(count($company_user) > 0) {
                //     $actions.='<a href="' . route("users.company",["company_id"=>$data->id]) . '" title="Users" ><i class="fas fa fa-user"></i></a>';
                //     // $actions.='<a href="' . route("users.company",["company_id"=>$data->id]) . '" title="Users" ><i class="fas fa fa-shopping-cart"></i></a>';
                // }
                // else {
                //     $actions.='<a href="' . route("users.create",["company_id"=>$data->id]) . '" title="Add Users" ><i class="fas fa fa-user-plus"></i></a>';
                // }    
                $company_roles=DB::table('company_users')
                        ->join('model_has_roles', 'company_users.user_id', '=', 'model_has_roles.model_id')
                        ->where('company_users.company_id','=',$data->id)
                        ->where('model_has_roles.role_id','=',2)
                        ->select('company_users.company_id as company_id','model_has_roles.role_id as role_id');
                // if(is_null($company_roles)){
                    // $actions.='<label class="btn btn-danger">Company admin is not assigned</label>';
                // }
                // $actions.=$company_roles;
                // $actions.='<i class="fa-sharp fa-solid fa-clipboard-check"></i>';
                // $actions.='<a href="' . route("company.dashboard",["company_id"=>$data->id]) . '" title="Company Dashboard" > Select</a>';
                // $actions.='<a href="' . route("company.dashboard",["company"=>$data->id]) . '" title="Company Dashboard" > Select</a>';
                $actions.='';
                $actions.='<a href="' . route("company.dashboard",["company"=>$data->id]) . '" title="Company Dashboard"><i class="fa fa-hand-pointer-o" aria-hidden="true"></i></a>';
                // $actions.='<small class="badge badge-warning"><a href="' . route("company.dashboard",["company"=>$data->id]) . '" title="Company Dashboard" style="color:#fff">Select</a></small>';
                // $actions.='<small class="badge badge-warning"><i class="far fa-clock"></i> 1 day</small>';
                
                $actions.='</div>';
                // $actions.='<small class="badge badge-warning"><i class="far fa-clock"></i> 1 day</small>';
                return $actions;

                
            });

            $datatables = $datatables->escapeColumns([])->toJson();
                return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in CompanyController userList :- " . $e->getMessage());
        } 
    }
    
    public function companyName(Request $request)
    {
        $company_id     = $request['company_id'];
        $company        = Company::find($company_id);
        $response       = array("company_name" => $company->company_name, "company_id" => $company_id);
        Session::put('temp_company',$company->id);  
        echo json_encode($response);
        return;
    }


    public function validateCompanyMail(Request $request)
    {
        $company_email      = $request->input('company_email');
        $company_id         = $request->input('company_id');
        
        $email_check        = Company::where(['email' => $company_email,])->where('id', '!=', $company_id)->first();
        $response           = [];
        if($email_check){
            $response['status']         = false;
            $response['msg']            = 'The email has already been taken.';
        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }

    public function validateNewCompanyMail(Request $request)
    {
        $company_email      = $request->input('company_email');
        
        $email_check        = Company::where('email', $company_email)->first();
        $response           = [];
        if($email_check){
            $response['status']         = false;
            $response['msg']            = 'The email has already been taken.';
        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }

    public function fetchCompanyLogo(Request $request) {
        $company_id= $request['company_id'];
        $company_image = Company::where('id',$company_id)->select('company_logo')->first();
        $image_url = url('/').'/storage/app/companies/'.$company_image->company_logo;
        echo $image_url;
    }

    public function fetchStateList(Request $request) {
        $country_id                     = $request->input('country_id');
        $company_id                     = $request->input('company_id');
        $company                        = Company::find($company_id);

        $states = DB::table('states')->where('country_id',$country_id)->select('id','name')->get();
        $response                       = [];

        if(count($states)>0){
            $state_select = '<strong><label>State :</label></strong>';
            $state_select .= '<select class="form-control form-control-sm" id="state" name="state">';
            foreach($states as $state) {
                $state_select .= '<option value="'.$state->id.'"';
                if($company!=null) {
                    if($state->id == $company->state) {
                        $state_select .= 'selected';
                    }
                }
                $state_select .= '>'.$state->name.'</option>';
            }
            $state_select .= '</select>';

            $response['status']         = true;
            $response['msg']            = $state_select;
        }else{
            $response['status']         = false;
        }

        echo json_encode($response);
        return;
    }

    public function validateNewCompanyCode(Request $request) {
        $company_code                   = $request->input('company_code');
        $company_code_check             = Company::where('company_code', $company_code)->withTrashed()->first();
        $response                       = [];
        if($company_code_check){
            $response['status']         = false;
            $response['msg']            = 'The company code has already been taken.';
        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }

    public function validateCompanyCode(Request $request) {
        $company_code                   = $request->input('company_code');
        $company_id                     = $request->input('company_id');
        $company_code_check             = Company::where('company_code', $company_code)->where('id', '!=', $company_id)->withTrashed()->first();
        $response                       = [];
        if($company_code_check){
            $response['status']         = false;
            $response['msg']            = 'The company code has already been taken.';
        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }
}

