<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Session;
use App\Models\Company;
use App\Models\User;
use App\Models\CompanyUser;
use App\Models\Categories;
use App\Models\Chat;
use App\Models\Order;
use App\Models\OrderBatch;
use App\Models\Product;
use App\Models\ScannedProduct;
use App\Models\OrderProductDetil;
use Auth;

use Log;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        $user                                         = Auth::user();
        $company_user                                 = CompanyUser::where('user_id',$user->id)->first();
        $company                                      = Company::where('id',$company_user->company_id)->first();
        if(Session::get('company')){
          $company_id                                 = Session::get('company');   
          $company                                    = Company::where('id',$company_id)->first();  
        }else {
            Session::put('company', $company->id);
            Session::put('company_name',$company->company_name);
            Session::put('company_logo',$company->company_logo);
            Session::put('logged_in_company',$company->id);
            return redirect('home');
        }
        $dashboard_data                               = [];

        if($company->id == 1){
          $dashboard_data['total_companies']          = Company::get()->count();
          $dashboard_data['app_users_count']          = User::where('user_type' , 1)->get()->count();  
          $dashboard_data['order_count']              = Order::where('print_status',1)->get()->count();  



          $dashboard_data['total_labels']             = OrderBatch::get();
          $dashboard_data['total_printed_labels']     = OrderBatch::where('label_status_1' , 2)->get();
          $dashboard_data['total_un_printed_labels']  = OrderBatch::where('label_status_1' , 1)->get();

          
          $chat_list                                  = Chat::with('from')->orderBy('updated_at','desc')->get();
          $unreaded_chat_count                        = 0;
          foreach ($chat_list as $key => $chat) {
              if(sizeof($chat->unreadedMessagesAppUser) > 0){
                  $unreaded_chat_count++;
              }              
          }
          $dashboard_data['chat_count']                 = $unreaded_chat_count;

          $dashboard_data['latest_orders']              = Order::where('print_status',1)->limit(5)->orderBy('id','desc')->get();
          $dashboard_data['latest_products']            = Product::limit(5)->orderBy('id','desc')->get();
        }else{
            //dd($company_id );
            $dashboard_data['total_companies']          = Company::get()->count();
            $dashboard_data['app_users_count']          = User::where('user_type' , 1)->get()->count();  
            $dashboard_data['order_count']              = Order::where(['print_status' => 1 , 'company_id' => $company_id])->get()->count();  

            $dashboard_data['total_labels']             = 0;
            $dashboard_data['total_printed_labels']     = 0;
            $dashboard_data['total_un_printed_labels']  = 0;

            $order_list                                 = Order::where('company_id', $company_id)->get();

            foreach($order_list as $order){
              $order_details                            = OrderProductDetil::where('order_id',$order->id)->get();

              foreach($order_details as $order_detail){
                if( $order_detail->batches->isNotEmpty()){
                    foreach($order_detail->batches as $batches){
                      $dashboard_data['total_labels']   += $batches->label_count;
                      if($batches->label_status_1 == 2){
                        $dashboard_data['total_printed_labels'] += $batches->label_count;
                      }

                      if($batches->label_status_1 == 1){
                        $dashboard_data['total_un_printed_labels'] += $batches->label_count;
                      }
                    }
                }
              }
            }
            
            $chat_list                                  = Chat::with('from')->where('company_id', $company_id)->orderBy('updated_at','desc')->get();
            $unreaded_chat_count                        = 0;
            foreach ($chat_list as $key => $chat) {
                if(sizeof($chat->unreadedMessagesAppUser) > 0){
                    $unreaded_chat_count++;
                }              
            }

            $dashboard_data['latest_orders']              = Order::where('company_id',$company_id)->limit(5)->orderBy('id','desc')->get();
            $dashboard_data['latest_products']              = Product::where('company_id',$company_id)->limit(5)->orderBy('id','desc')->get();


        }

        return view('home',compact('company', 'dashboard_data'));
        // } 
    }

    public function setCompany(Request $request) {
        if($request['company']!=null)   {
            $company_id               = $request['company'];
            $company                  = Company::where('id',$company_id)->first();
            if(Session::get('company')){
                Session::forget('company');
                Session::forget('company_name');
                Session::forget('company_logo');
                Session::put('company', $company_id);
                Session::put('company_name',$company->company_name);
                Session::put('company_logo',$company->company_logo);
            }else{
                Session::put('company', $company_id);
                Session::put('company_name',$company->company_name);
                Session::put('company_logo',$company->company_logo);
            } 
        }
        return redirect('home'); 
    }

    public function resetCompany(Request $request) {
        try {
            if(Session::get('company')){
                Session::forget('company');
                Session::forget('company_name');
                $company=Company::where('id',1)->first();
                session(['company'=>1,'company_name' => $company->company_name,'company_logo' => $company->company_logo]);
        
            }
            return redirect('companies');
        }
        catch (\Exception $e) {
            Log::error("Error in HomeController setCompany :- " . $e->getMessage());
        }
    }


    public function testDatatable()
    {
        $session_company    = Session::get('company');
        $data               = Categories::orderBy('id','ASC')->where('company_id',$session_company)->paginate(10);
        $companies          = Company::pluck('company_name','id')->all();

        return view('datatable', compact('data','companies'));
    }
}
