<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Product;
use App\Models\Categories;
use App\Models\Company;
use App\Models\CompanyUser;
use App\Models\ProductWarranty;
use Spatie\Permission\Models\Permission;
use Illuminate\Support\Facades\Log;
use App\Models\WarrantyPart;
use Auth;
use DB;
use Session;


class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    function __construct()
    {
         $this->middleware('permission:products-list|products-create|products-edit|products-delete', ['only' => ['index','store']]);
         $this->middleware('permission:products-create', ['only' => ['create','store']]);
         $this->middleware('permission:products-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:products-delete', ['only' => ['destroy']]);
    }

    public function index(Request $request)
    {
        $session_company=Session::get('company');

        if(Auth::user()->id == 1){
            if(Session::get('company')!=null && Session::get('company')!=1){
                // $company=Session::get('company');
                $products = Product::orderBy('id','DESC')->where('company_id',$session_company)->paginate(10);
            }
            else {
                $products = Product::orderBy('id','DESC')->paginate(10);
            }
            
        }
        else {
            // $company=Session::get('company');
            // $company=CompanyUser::where('user_id',Auth::user()->id)->first();
            $products = Product::orderBy('id','DESC')->where('company_id',$session_company)->paginate(10);
        }

        $companies = Company::pluck('company_name','id')->all();

        if($request['company_id']!=null) {
            $company_id=$request['company_id'];
            // $company=null;
        }
        elseif($request['company_id']==null && $session_company==null) {
                $company_id=1;
                // $company=null;
            }
        elseif($session_company!=null){
            $company_id=$session_company;
        }
        else {
            $company_id=$request['company_id'];
            // $company=Company::where('id',$company_id)->first();
        }

        return view('products.index',compact('products','companies','company_id'))
            ->with('i', ($request->input('page', 1) - 1) * 10);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $session_company=Session::get('company');
        
        if($session_company != 1){
            $companies      = DB::table('company_users')->where('user_id',Auth::user()->id)->first();
            if($companies)
            {
                $company        =Company::find($companies->company_id);
            }   
            $companies          = Company::select('id','company_name')->where('id', $company->id)->get();

            $company_categories = DB::table('company_categories')->where('company_id',$company->id)->get();
            // $temp_category_id   = [];
            // foreach($company_categories as $c_c){
            //     $temp_category_id[] = $c_c->category_id;
            // }

            // $categories         = Categories::where('company_id', $session_company)->select('id','category')->whereIn('id',$temp_category_id)->get();
            
        }else{
            $companies          = Company::select('id','company_name')->get();
            // $categories         = Categories::select('id','category')->get();
            
        }

        $categories = Categories::with('subCategory')->where(['company_id' => $session_company,'parent_id' => 0 ])->get();
        $category = Categories::with('subCategory')->where(['company_id' => $session_company,'parent_id' => 0])->first();
        if($category!=null) {
            $subcategories = Categories::where(['parent_id' => $category->id ])->get();
        }
        else {
            $subcategories = [];
        }
        $last_product_id                    = Product::orderBy('id', 'DESC')->first();
        if($last_product_id){
            if($last_product_id ->id < 10){
                $next_product_code              = 'P00'.$last_product_id ->id;
            }elseif($last_product_id ->id >= 10 && $last_product_id ->id < 100){
                $next_product_code              = 'P0'.$last_product_id ->id+1;
            }else{
               $next_product_code               = 'P'.$last_product_id ->id+1; 
            }
        }else{
            $next_product_code              = 'P001';
        }
       
        return view('products.create', compact('companies','categories','session_company','subcategories','next_product_code'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
                'product_name'  => 'required',
                'description'   => 'required',
                'status'        => 'required',
                'url'           => 'required|url',
                'purchase_link' => 'required|url',
                'image'         => 'required|mimes:jpeg,jpg,png,gif|max:10000',
                'brand_name'    => 'required',
                // 'logo'          => 'required|mimes:jpeg,jpg,png,gif|max:10000',
                // 'company'       => 'required',
                'category'      => 'required',
                'product_label' => 'required|mimes:jpeg,jpg,png,gif|max:10000',
                'product_label2' => 'required|mimes:jpeg,jpg,png,gif|max:10000',
        ]);

        $company_id                     = Session::get('company');
        $product                        = new Product;
        $product->product_name          = $request->input('product_name');

        $last_product_id                    = Product::orderBy('id', 'DESC')->first();
        if($last_product_id){
            if($last_product_id ->id < 10){
                $next_product_code              = 'P00'.$last_product_id ->id;
            }elseif($last_product_id ->id >= 10 && $last_product_id ->id < 100){
                $next_product_code              = 'P0'.$last_product_id ->id+1;
            }else{
               $next_product_code               = 'P'.$last_product_id ->id+1; 
            }
        }else{
            $next_product_code              = 'P001';
        }

        $product->product_code          = $next_product_code;
        $product->model_number          = $request->input('model_number');
        if($company_id==1) {
            $product->company_id        = $request->input('company');
        }else{
            $product->company_id        = $company_id;
        }
        $product->brand_name            = $request->input('brand_name');
        $product->category_id           = $request->input('category');
        $product->description           = $request->input('description');
        $product->status                = $request->input('status');

        $product_image_name             = time().'Product.'.$request->image->extension();
        // $product_logo_name              = time().'Logo.'.$request->logo->extension();
        $product_label_name             = time().'Label.'.$request->product_label->extension();
        $product_label2_name            = time().'Label2.'.$request->product_label->extension();

        if($request->image->storeAs('public/app/images', $product_image_name)){
            $product->product_image     = $product_image_name;
        }
        // if($request->logo->storeAs('public/app/logos', $product_logo_name)){
        //     $product->product_logo      = $product_logo_name;
        // }

        if($request->product_label->storeAs('public/app/product_label', $product_label_name)){
            $product->product_label      = $product_label_name;
            $product->qrcode_size        = $request->input('qrcode_size');
            $product->qrcode_top         = $request->input('qrcode_top');
            $product->qrcode_left        = $request->input('qrcode_left');
            $product->qrcode_ratio       = $request->input('productLabel_ratio');

        }

        if($request->product_label2->storeAs('/product_label2', $product_label2_name)){
            $product->product_label2     = $product_label2_name;
            $product->qrcode_size2       = $request->input('qrcode_size2');
            $product->qrcode_top2        = $request->input('qrcode_top2');
            $product->qrcode_left2       = $request->input('qrcode_left2');
            $product->qrcode_ratio2      = $request->input('productLabel_ratio2');

        }

        $product->quantity           = $request->input('quantity');
        $product->quantity_measured  = $request->input('quantity_measured');
        $product->expiry_period      = $request->input('expiry_period');
        $product->number_in_pack     = $request->input('number_in_pack');
        $product->packing_method     = $request->input('packing_method');

        $product->url                = $request->input('url');
        $product->purchase_link      = $request->input('purchase_link');

        $product->auth_prod_name     = $request->input('auth_prod_name');
        $product->auth_done_by       = $request->input('auth_done_by');

        $product->url                   = $request->input('url');

        if($product->save()){
            $warranty                       = new ProductWarranty;

            $warranty->product_id           = $product->id;
            $warranty->waranty_status       = $request->input('waranty_status');
            $warranty->promotion_status     = $request->input('promotion_status');


            $warranty->warranty_type        = null;
            $warranty->full_product         = null;
            $warranty->s_warranty_count     = null;
            $warranty->s_warranty_duration  = null;
            // $warranty->service_status       = null;
            // $warranty->replacement_status   = null;
            $warranty->r_warranty_count     = null;
            $warranty->r_warranty_duration  = null;

            if($warranty->waranty_status=="Yes") {
                $warranty->warranty_type   = $request->input('warranty_type');
                if($warranty->warranty_type=="1"){
                    $warranty->full_product   = $request->input('full_product');
                    if($warranty->full_product=="Yes"){
                        $this->validate($request, [
                            's_warranty_count'  => 'required',
                            // 'r_warranty_count'  => 'required',
                        ]);
                        $warranty->s_warranty_count   = $request->input('s_warranty_count');
                        if($warranty->s_warranty_count>0) {
                            $warranty->s_warranty_duration   = $request->input('s_warranty_duration');
                        }
                    }elseif($warranty->full_product=="No"){
                        $this->validate($request, [
                            'part_name'         => 'required',
                            'part_name.*'       => 'string',
                            'part_warranty'     => 'required',
                            'part_warranty.*'   => 'numeric',
                        ]);
                        $warranty_parts         = $request->input('part_name');
                        $part_duration          = $request->input('part_warranty');
                        foreach ($warranty_parts as $key => $part_name) {
                            if($part_name != ''){
                                $new_warranty_part                          = new WarrantyPart;
                                $new_warranty_part->product_id              = $product->id;
                                $new_warranty_part->part_name               = $part_name;
                                $new_warranty_part->part_warranty_count     = $part_duration[$key];
                                $new_warranty_part->part_warranty_duration  = 'Months';
                                $new_warranty_part->save();
                            }
                        }
                    }
                }
                // $warranty->replacement_status   = $request->input('replacement_status');
                if($warranty->warranty_type=="2"){
                    $this->validate($request, [
                        'r_warranty_count'  => 'required',
                    ]);
                    $warranty->r_warranty_count   = $request->input('r_warranty_count');
                    if($warranty->r_warranty_count>0) {
                        $warranty->r_warranty_duration   = $request->input('r_warranty_duration');
                    }
                }
            }
            $warranty->save();
        }
        return redirect()->route('products.index')->with('success', 'Product Added Successfully!');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $session_company    = Session::get('company');
        $product    = Product::with('company','category','partWarranty')->where('id',$id)->first();
        $company    = Company::where(['id' => $product->company_id ])->first();
        $category   = Categories::where(['id' => $product->category_id ])->first();
        if($product->subcategory_id!=null) {
            $subcategory   = Categories::where(['id' => $product->subcategory_id ])->first();
        }
        else {
            $subcategory   = [];
        }

        $warranty_details   = ProductWarranty::where('product_id',$id)->first();

        return view('products.show',compact('company','category','subcategory','product','warranty_details'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $session_company=Session::get('company');
        $product                    = Product::with('company','category','partWarranty')->where('id',$id)->first();
        if($session_company != 1){
            $companies              = Company::where('id', $session_company)->select('id','company_name')->get();
        }else{
            $companies              = Company::select('id','company_name')->get();
        }
        // $categories                 = Categories::where('company_id', $session_company)->select('id','category')->get(); 
        
        $categories = Categories::with('subCategory')->where(['company_id' => $product->company_id, 'parent_id'=>0 ])->get();
        $company_img = Company::where('id', $product->company_id)->select('company_name','company_logo')->first();

        $subcategories = Categories::where(['parent_id' => $product->category_id ])->get();

        $warranty_details           = ProductWarranty::where('product_id',$id)->first();

        return view('products.edit', compact('companies','company_img','categories','subcategories','product','warranty_details'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $company_id=Session::get('company');
        $this->validate($request, [
                'product_name'  => 'required',
                'description'   => 'required',
                'status'        => 'required',
                'url'           => 'required|url',
                'purchase_link' => 'required|url',
                'brand_name'    => 'required',
                // 'company'       => 'required',
                'category'      => 'required',
        ]);
        if($request->hasFile('image')){
            $this->validate($request, [
                'image'         => 'mimes:jpeg,jpg,png,gif|max:10000',
                'logo'          => 'mimes:jpeg,jpg,png,gif|max:10000',
            ]);
        }
        if($request->hasFile('logo')){
            $this->validate($request, [
                'logo'          => 'mimes:jpeg,jpg,png,gif|max:10000',
            ]);
        }

        if($request->hasFile('product_label')){
            $this->validate($request, [
                'product_label'          => 'mimes:jpeg,jpg,png,gif|max:10000',
            ]);
        }

        $product                        = Product:: find($id);
        $product->product_name          = $request->input('product_name');
        //$product->product_code          = $request->input('product_code');
        $product->model_number          = $request->input('model_number');
        if($company_id==1) {
            $product->company_id        = $request->input('company');
        }
        else {
            $product->company_id        = $company_id;
        }
        $product->brand_name            = $request->input('brand_name');
        $product->category_id           = $request->input('category');
        $product->subcategory_id        = $request->input('subcategory');
        $product->description           = $request->input('description');
        $product->status                = $request->input('status');

        // if($request->hasFile('image')){
        //     $product_image_name         = time().'Product.'.$request->image->extension();
        //     if($request->image->storeAs('images', $product_image_name)){
        //         $product->product_image = $product_image_name;
        //     }
        // }
        // if($request->hasFile('logo')){
        //     $product_logo_name          = time().'Logo.'.$request->logo->extension();
        //     if($request->logo->storeAs('logos', $product_logo_name)){
        //         $product->product_logo  = $product_logo_name;
        //     }
        // }
        // if($request->hasFile('product_label')){
        //     $product_label_name          = time().'Label.'.$request->product_label->extension();
        //     if($request->product_label->storeAs('product_label', $product_label_name)){
        //         $product->product_label  = $product_label_name;
        //     }
        // }
        
        if($request->hasFile('image')){
            $product_image_name         = time().'Product.'.$request->image->extension();
            if($request->file('image')->storeAs('public/app/images',$product_image_name)){
                $product->product_image = $product_image_name;
            }
        }

        if($request->hasFile('logo')){
            $product_logo_name          = time().'Logo.'.$request->logo->extension();
            if($request->logo->storeAs('public/app/logos', $product_logo_name)){
                $product->product_logo  = $product_logo_name;
            }
        }
        if($request->hasFile('product_label')){
            $product_label_name          = time().'Label.'.$request->product_label->extension();
            if($request->product_label->storeAs('public/app/product_label', $product_label_name)){
                $product->product_label  = $product_label_name;
            }
            
        }

        if($request->hasFile('product_label2')){
            $product_label2_name          = time().'Label2.'.$request->product_label2->extension();
            if($request->product_label2->storeAs('public/app/product_label2', $product_label2_name)){
                $product->product_label2  = $product_label2_name;
            }
            
        }

        $product->qrcode_size        = $request->input('qrcode_size');
        $product->qrcode_top         = $request->input('qrcode_top');
        $product->qrcode_left        = $request->input('qrcode_left');
        $product->qrcode_ratio       = $request->input('productLabel_ratio');




        $product->qrcode_size2       = $request->input('qrcode_size2');
        $product->qrcode_top2        = $request->input('qrcode_top2');
        $product->qrcode_left2       = $request->input('qrcode_left2');
        $product->qrcode_ratio2      = $request->input('productLabel_ratio2');

        $product->quantity           = $request->input('quantity');
        $product->quantity_measured  = $request->input('quantity_measured');
        $product->expiry_period      = $request->input('expiry_period');
        $product->number_in_pack     = $request->input('number_in_pack');
        $product->packing_method     = $request->input('packing_method');

        $product->url                = $request->input('url');
        $product->purchase_link      = $request->input('purchase_link');

        $product->auth_prod_name     = $request->input('auth_prod_name');
        $product->auth_done_by       = $request->input('auth_done_by');

        $product->save();

        // $product->purchase_link      = $request->input('purchase_link');

        $warranty                       = ProductWarranty::where('product_id',$product->id)->first();

        $warranty->waranty_status       = $request->input('waranty_status');
        $warranty->promotion_status     = $request->input('promotion_status');


        $warranty->warranty_type        = null;
        $warranty->full_product         = null;
        $warranty->s_warranty_count     = null;
        $warranty->s_warranty_duration  = null;
        // $warranty->service_status       = null;
        // $warranty->replacement_status   = null;
        $warranty->r_warranty_count     = null;
        $warranty->r_warranty_duration  = null;

        

        if($warranty->waranty_status=="Yes") {
            $warranty->warranty_type   = $request->input('warranty_type');
            if($warranty->warranty_type=="1"){
                $warranty->full_product   = $request->input('full_product');
                if($warranty->full_product=="Yes"){
                    $this->validate($request, [
                        's_warranty_count'  => 'required',
                        // 'r_warranty_count'  => 'required',
                    ]);
                    $warranty->s_warranty_count   = $request->input('s_warranty_count');
                    if($warranty->s_warranty_count>0) {
                        $warranty->s_warranty_duration   = $request->input('s_warranty_duration');
                    }
                    WarrantyPart::where('product_id',$product->id)->delete();
                }elseif($warranty->full_product=="No"){
                    $this->validate($request, [
                        'part_name'         => 'required',
                        'part_warranty'     => 'required',
                    ]);
                    $warranty_parts         = $request->input('part_name');
                    $part_duration          = $request->input('part_warranty');
                    WarrantyPart::where('product_id',$product->id)->delete();

                    foreach ($warranty_parts as $key => $part_name) {
                        if($part_name != ''){
                            $new_warranty_part                          = new WarrantyPart;
                            $new_warranty_part->product_id              = $product->id;
                            $new_warranty_part->part_name               = $part_name;
                            $new_warranty_part->part_warranty_count     = $part_duration[$key];
                            $new_warranty_part->part_warranty_duration  = 'Months';
                            $new_warranty_part->save();
                        }
                    }
                }
            }
            // $warranty->replacement_status   = $request->input('replacement_status');
            if($warranty->warranty_type=="2"){
                $this->validate($request, [
                    'r_warranty_count'  => 'required',
                ]);
                
                $warranty->r_warranty_count   = $request->input('r_warranty_count');
                if($warranty->r_warranty_count>0) {
                    $warranty->r_warranty_duration   = $request->input('r_warranty_duration');
                }
            }   
        }
        $warranty->save();
        return redirect()->route('products.index')->with('success', 'Product Details Updated Successfully!');

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            Product::where('id',$id)->delete();
                return redirect()->route('products.index')
                        ->with('success','Product Deleted Successfully');
        }
        catch (\Exception $e) {
            Log::error("Error in Product deletion :- " . $e->getMessage());
        } 
    }

    public function productList(Request $request)
    {
        try {
            $Products = new Product;
            if(!$request->input('company_id'))   {
                $company = 0;
                }
            else {
                $company = $request['company_id'];
            }
            $company_id     = $company;
            $result         = $Products->get_details_product($company_id);
            $data           = collect($result);

      
            $datatables = datatables()->of($data)->addColumn('sl_no', function () {
                return '';
            });

            $datatables = $datatables->addColumn('product_name', function ($data) {
                $product_name = $data->product_name;
                $view = '<span  style="display:none;">'.$product_name.'</span><a href="' . route("products.show", $data->id) . '" title="Show" class="De_view">'.$product_name.'</i>';
                return $view;
            });


            $datatables = $datatables->addColumn('category', function ($data) {
                $category = Categories::find($data->category_id);
                if($category){
                    return $category->category;
                }else{
                    return 'No Category Choosed';
                }
  
            });

            
            $datatables = $datatables->addColumn('actions', function ($data) {
                $user = Auth()->user();
                $actions='<div class="action-icons">';
                if($user->can('products-edit')){
                    $actions.='<a href="' . route("products.edit", $data->id) . '" title="Edit" ><i class="fas fa-edit"></i></a>';
                }
                if($user->can('products-delete')){
                    $actions.='<button type="button" class="btn btn-xs btn-flat show_confirm delete-btn" data-bs-toggle="modal" data-bs-target="#deleteModal'. $data->id .'" data-product-id="'. $data->id .'"  title="Delete"><i class="fa fa-trash-o" aria-hidden="true"> </i></button>';
                
                
                    $actions.='<div class="modal fade" id="deleteModal'. $data->id .'" tabindex="-1" aria-labelledby="deleteModalLabel" aria-hidden="true">
                    <div class="modal-dialog">
                    <div class="modal-content">
                    <div class="modal-header">
                        <h5 class="modal-title" id="deleteModalLabel">Delete Confirmation</h5>
                        </div>
                        <div class="modal-body">
                        Are you sure you want to delete this product?
                        </div>
                        <div class="modal-footer">
                        <button type="button" class="btn btn-secondary modal-btn" data-dismiss="modal">Cancel</button>
                        <form method="POST" action="'.route("products.destroy", $data->id).'">
                        <input name="_method" type="hidden" value="DELETE">
                        <input type="hidden" name="_token" value="'.csrf_token().'" />
                            <button type="submit" class="btn btn-danger text-right modal-btn">Delete</button>
                        </form>
                        </div>
                    </div>
                    </div>
                    </div>';
                }
                
                $actions.='</div>';
                return $actions;

             
            });

            $datatables = $datatables->escapeColumns([])->toJson();
                return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in productController productList :- " . $e->getMessage());
        } 


    }

    public function validateNewProductCode(Request $request) {
        $product_code                   = $request->input('product_code');
        $product_code_check             = Product::where('product_code', $product_code)->withTrashed()->first();
        $response                       = [];
        if($product_code_check){
            $response['status']         = false;
            $response['msg']            = 'The product code has already been taken.';
        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }

    public function validateProductCode(Request $request) {
        $product_code                   = $request->input('product_code');
        $product_id                     = $request->input('product_id');
        $product_code_check             = Product::where('product_code', $product_code)->where('id', '!=', $product_id)->withTrashed()->first();
        $response                       = [];
        if($product_code_check){
            $response['status']         = false;
            $response['msg']            = 'The product code has already been taken.';
        }else{
            $response['status']         = true;
        }
        echo json_encode($response);
        return;
    }
}
