<?php
    
namespace App\Http\Controllers;
    
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Company;
use App\Models\Categories;
use App\Models\CompanyCategory;
use App\Models\CompanyUser;
use App\Models\ContactPerson;
use App\Models\ScannedProduct;
use App\Models\Product;
use App\Models\OrderBatch;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Traits\HasRoles;
use DB;
use Hash;
use Session;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
use Str;
    
class ReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
         $this->middleware('permission:company-list|company-create|company-edit|company-delete', ['only' => ['index','store']]);
         $this->middleware('permission:company-create', ['only' => ['create','store']]);
         $this->middleware('permission:company-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:company-delete', ['only' => ['destroy']]);
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        //
    }

    public function companyReport(Request $request)
    {
        try {            
            return view('reports.companies');
        }
        catch (\Exception $e) {
            Log::error("Error in ReportController companyReport :- " . $e->getMessage());
        } 
    }

    public function companyReportList(Request $request)
    {
        try {
            $Company = new Company;

            $company_type   = $request['company_type'];
            $start_date     = $request['start_date'];
            $end_date       = $request['end_date'];
                
            $result = $Company->getReportsCompany($company_type, $start_date, $end_date);
            $data = collect($result);

            
            $datatables = datatables()->of($data)->addColumn('sl_no', function () {
                return '';
            });

            $datatables =$datatables->addColumn('company_name', function ($data) {
                $company_name = $data->company_name;
                $view = '<span>'.$company_name.'</span>';
                return $view;
            });

            $datatables =$datatables->addColumn('company_type', function ($data) {
                $view = ucfirst(strtolower($data->company_type));
                return $view;
            });

            $datatables                 = $datatables->addColumn('created_at', function ($data) {
                return date("d-M-Y", strtotime($data->created_at));
            });


           

            $datatables = $datatables->escapeColumns([])->toJson();
                return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in ReportController companyReportList :- " . $e->getMessage());
        } 
    }


    public function productReport(Request $request)
    {
        try {            
            $categories         = Categories::where('parent_id', 0)->orderBy('category','ASC')->get();
            $companies          = Company::orderBy('company_name','ASC')->get();
            return view('reports.products',compact('categories','companies'));
        }
        catch (\Exception $e) {
            Log::error("Error in ReportController productReport :- " . $e->getMessage());
        } 
    }

    public function productReportList(Request $request)
    {
        try {
            $product                    = new Product;
            $company_id                 = $request['product_company'];
            $category_id                = $request['product_category'];
            $status                     = $request['product_status'];
            $start_date                 = $request['start_date'];
            $end_date                   = $request['end_date'];
            $result                     = $product->getReports($company_id,$category_id,$status, $start_date, $end_date);
            $data                       = collect($result);
            $datatables                 = datatables()->of($data)->addColumn('sl_no', function () {
                return '';
            });

            $datatables                 = $datatables->addColumn('product_name', function ($data) {
                return  '<span><a href="' . route("products.show", $data->id) . '" title="Show" class="De_view">'.$data->product_name.'</a></span>';
            });

            $datatables                 = $datatables->addColumn('company_type', function ($data) {
                return ucfirst(strtolower($data->company_type));
            });
            /*
            $datatables                 = $datatables->addColumn('product_image', function ($data) {
                return ucfirst(strtolower($data->company_type));
            });
            **/
            
            $datatables                 = $datatables->addColumn('description', function ($data) {
                return Str::limit($data->description,20);
            });

            $datatables                 = $datatables->addColumn('product_model', function ($data) {
                return ucfirst(strtolower($data->model_number));
            });

            $datatables                 = $datatables->addColumn('warranty', function ($data) {

                if($data->productWarranty){
                    return $data->productWarranty->waranty_status;
                }else{
                    return 'No';
                }
            });

            $datatables                 = $datatables->addColumn('promotion', function ($data) {

                if($data->productWarranty){
                    return $data->productWarranty->promotion_status;
                }else{
                    return 'No';
                }
            });

            $datatables                 = $datatables->addColumn('created_at', function ($data) {
                return date("d-M-Y", strtotime($data->created_at));
            });
            $datatables                 = $datatables->escapeColumns([])->toJson();
            return $datatables;
        }catch (\Exception $e) {
            Log::error("Error in ReportController productReportList :- " . $e->getMessage());
        } 
    }

    public function batchReport(Request $request)
    {
        try{
            return view('reports.batches');
        }catch (\Exception $e) {
            Log::error("Error in ReportController batchReport :- " . $e->getMessage());
        } 
    }

    public function batchReportList(Request $request)
    {
        try {
            $batch                      = new OrderBatch;
            $start_date                 = $request['start_date'];
            $end_date                   = $request['end_date'];
            $result                     = $batch->getReports($start_date, $end_date);
            $data                       = collect($result);
            $datatables                 = datatables()->of($data)->addColumn('sl_no', function () {
                return '';
            });

            $datatables                 = $datatables->addColumn('product_name', function ($data) {
                return '<span><a href="' . route("products.show", $data->productOrderDetails->products->id) . '" title="Show" class="De_view">'.$data->productOrderDetails->products->product_name.'</a></span>';
            });

            $datatables                 = $datatables->addColumn('product_code', function ($data) {
                return '<span>'.$data->productOrderDetails->products->product_code.'</span>';
            });

            $datatables                 = $datatables->addColumn('batch_no', function ($data) {
                return ($data->batch_id);
            });

            $datatables                 = $datatables->addColumn('order_id', function ($data) {
                return '<a href="'.route('orders.details',$data->productOrderDetails->orderDetails->id).'" title="Details">#'.($data->productOrderDetails->orderDetails->id).'</a>';
            });

            $datatables                 = $datatables->addColumn('label_count', function ($data) {
                return $data->label_count;
            });

            $datatables                 = $datatables->addColumn('start_no', function ($data) {
                return $data->start_number;
            });

            $datatables                 = $datatables->addColumn('end_no', function ($data) {
                return $data->start_number + ($data->label_count - 1);
            });       
            $datatables                 = $datatables->addColumn('created_at', function ($data) {
                return date("d-M-Y", strtotime($data->created_at));
            });
            $datatables                 = $datatables->escapeColumns([])->toJson();
            return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in ReportController batchReportList :- " . $e->getMessage());
        } 
    }
    
    
    public function scannedProductsReport(Request $request)
    {
        try{
            return view('reports.scanned-product');
        }catch (\Exception $e) {
            Log::error("Error in ReportController scannedProductsReport :- " . $e->getMessage());
        } 
    }

    public function scannedProductsReportList(Request $request)
    {
        try {
            $scanned_product            = new ScannedProduct;
            $start_date                 = $request['start_date'];
            $end_date                   = $request['end_date'];
            $result                     = $scanned_product->getReports($start_date, $end_date);
            $data                       = collect($result);
            $datatables                 = datatables()->of($data)->addColumn('sl_no', function () {
                                                return '';
                                            });
            $datatables                 = $datatables->addColumn('product_name', function ($data) {
                return                   '<span><a href="' . route("products.show", $data->product->id) . '" title="Show" class="De_view">'.$data->product->product_name.'</a></span>';
            });

            $datatables                 = $datatables->addColumn('product_code', function ($data) {
                return                   '<span>'.$data->product->product_code.'</span>';
            });

            $datatables                 = $datatables->addColumn('user_name', function ($data) {
                if($data->user){
                    return               (strtoupper($data->user->name));
                }
                return 'User Removed';
            });

            $datatables                 = $datatables->addColumn('company_name', function ($data) {
                return                   strtoupper($data->company->company_name);
            });

            $datatables                 = $datatables->addColumn('batch_number', function ($data) {
                $decoded_qr_code        = explode('-',$data->code);
                if(isset($decoded_qr_code[3])){
                    return $decoded_qr_code[3];
                }else{
                    return '-- --';
                }                
            });           

            $datatables                 = $datatables->addColumn('authorization', function ($data) {
                if($data->authorization ){
                    return '<i class="fas fa-thumbs-up fa-2xl" style="color:green"></i>';
                }else{
                    return '<i class="fas fa-thumbs-down fa-2xl" style="color:red"></i>';
                }
            });
                        
            $datatables                 = $datatables->addColumn('scanned_location', function ($data) {
                return                   '<a target="_blank" href="https://www.google.com/maps/@'.$data->latitude.','.$data->longitude.'"> <i class="fa fa-map-marker fa-2x" aria-hidden="true"></i></a>';
            });

            $datatables                 = $datatables->addColumn('created_at', function ($data) {
                return date("d-M-Y", strtotime($data->created_at));
            });
            $datatables                 = $datatables->escapeColumns([])->toJson();
            return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in ReportController scannedProductsReportList :- " . $e->getMessage());
        } 
    }

}

