<?php
    
namespace App\Http\Controllers;
    
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Categories;
use App\Models\Company;
use Spatie\Permission\Models\Role;
use DB;
use Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
use Session;
use Redirect;
    
class CatergoriesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
         $this->middleware('permission:category-list|category-create|category-edit|category-delete', ['only' => ['index','store']]);
         $this->middleware('permission:category-create', ['only' => ['create','store']]);
         $this->middleware('permission:category-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:category-delete', ['only' => ['destroy']]);
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $session_company    = Session::get('company');
        $data               = Categories::orderBy('id','ASC')->where('company_id',$session_company)->paginate(10);
        $companies          = Company::pluck('company_name','id')->all();

        if($request['company_id']!=null) {
            $company_id=$request['company_id'];
        }
        elseif($request['company_id']==null && $session_company==null) {
                $company_id=1;

            }
        elseif($session_company!=null){
            $company_id=$session_company;
        }
        else {
            $company_id=$request['company_id'];
        }

        return view('categories.index', compact('data','companies','company_id'));
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(Request $request)
    {
        try {
            // $categories = Categories::get();
            // return view('categories.create',compact('categories'));
            $session_company=Session::get('company');
            // if($session_company == 1){
            //     $categories = Categories::where('parent_id',0)->pluck('category','id');
            // }
            // else {
            //     $categories = Categories::where('parent_id',0)->where('company_id',$session_company)->pluck('category','id');  
            // }
            // if($session_company == 1){
            //     $categories = Categories::where('parent_id',0)->get();

            // }
            // else {
                $categories = Categories::where('parent_id',0)->where('company_id',$session_company)->get();  
            // }
            // echo '<pre>';
            // print_r($categories);
            // exit();

            
            if($request['company_id']==null){
                $companies = Company::where('id',$session_company)->pluck('company_name','id')->all();
            }
            else {
                $companies = Company::where('id',$request['company_id'])->pluck('company_name','id')->all();
            }

            return view('categories.create',compact('categories','companies'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController create :- " . $e->getMessage());
        } 
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // try {
            $this->validate($request, [
                'category' => 'required',
            ]);
        
            // $categories = Categories::create(['category' => $request->input('category')]);

            $categories = new Categories;
            // $categories->category   = $request->input('category');
            // $categories->company_id = Session::get('company');
            // $categories->parent_id  = 0;

            $categories->parent_id      = $request->input('parent_id');
            $categories->category       = $request->input('category');
            $categories->company_id     = $request->input('companies');

            $categories->save();

            return redirect()->route('categories.index',['company_id' => $categories->company_id])
                            ->with('success','Category created successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController store :- " . $e->getMessage());
        // } 
    
    }
    
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $category = Categories::find($id);
            return view('categories.show',compact('category'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController show :- " . $e->getMessage());
        } 
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, $id)
    {
        // try {
            $category_info = Categories::find($id);

            $session_company=Session::get('company');
            // if($session_company == 1){
            //     $categories = Categories::where('parent_id',0)->pluck('category','id');
            // }
            // else {
            //     $categories = Categories::where('parent_id',0)->where('company_id',$session_company)->pluck('category','id');  
            // }
            // if($session_company == 1){
            //     $categories = Categories::where('parent_id',0)->where('id','!=',$id)->get();
            // }
            // else {
                $categories = Categories::where('parent_id',0)->where('id','!=',$id)->where('company_id',$session_company)->get();  
            // }


            if($request['company_id']==null) {
                $companies = Company::where('id',$session_company)->pluck('company_name','id')->all();
            }
            else {
                $companies = Company::where('id',$request['company_id'])->pluck('company_name','id')->all();
            }

            return view('categories.edit',compact('category_info','categories','companies'));
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController edit :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // try {
            $this->validate($request, [
                'category' => 'required',
            ]);
            
            $categories = Categories::find($id);
            
            if($categories->parent_id==0 && $request->input('parent_id')!=0) {
                $sub_categories = Categories::where('parent_id',$id)->get();
                if(count($sub_categories)>0) {
                    return redirect()->route('categories.edit',$id)
                            ->with('error','Sorry, this category has subcategories. If you want to change the parent, you must first change the parent of its subcategories and try again.');
                }
            }
            $categories->category      = $request->input('category');
            $categories->company_id    = $request->input('companies');
            // $categories->parent_id  = 0;
            $categories->parent_id      = $request->input('parent_id');



            $categories->save();

            return redirect()->route('categories.index',['company_id' => $categories->company_id])
                            ->with('success','Category updated successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController update :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {

            $categories = Categories::find($id);

            if($categories->parent_id==0) {
                $sub_categories = Categories::where('parent_id',$id)->get();
                if(count($sub_categories)>0) {
                    DB::table("categories")->where('parent_id',$id)->delete();
                }
            }

            DB::table("categories")->where('id',$id)->delete();
                return redirect()->route('categories.index',['company_id' => $categories->company_id])
                        ->with('success','Category deleted successfully');
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController destroy :- " . $e->getMessage());
        } 
    }

    public function categoryList(Request $request) {
        try {
            $Categories = new Categories;

            if($request['company_id']==null)   {
                $company = 0;
                }
            else {
                $company = $request['company_id'];
            }
            $company_id=$company;
                
            $result = Categories::with('subCategory')->where(['company_id' => $company_id, 'parent_id' => 0 ])->get();
            $category   = collect($result);
            $datatables = datatables()->of($category)->addColumn('sl_no', function () {
                return '';
            });
            $datatables = $datatables->addColumn('category', function ($category) {
                if($category->subCategory->isEmpty()){
                    return '<div class="shadow-none category-main">
                                <div class="card-header">
                                    <h3 class="card-title">'.$category->category.'</h3>
                                </div>
                            </div>';
                }else{
                    
                    // $temp_html      = '<a href="javascript:void(0)" class="category-toggle">'.$category->category.'</a>';
                    $temp_html      = '<div class="card shadow-none category-main collapsed-card subcategory-list">
                                            <div class="card-header">
                                                <h3 class="card-title">'.$category->category.'<button type="button" class="btn btn-tool collapse-btn" data-card-widget="collapse" title="Collapse">
                                                <i class="fas fa-plus"></i>
                                                </button></h3>
                                                <div class="card-tools">
                                                
                                                </div>

                                                </div>                                            
                                            <div class="card-body" style="display:none">';
                    
                        $sub_key        = 0;
                        foreach($category->subCategory as $subcategory){
                            // if($sub_key == 0){
                                $temp_html  .= '<div class="subCate">';   
                            // }
                            $sub_key++;
                            $user               = Auth()->user();
                            
                            $temp_html          .= '<div class="Cate-Slno">'.$sub_key.'</div><div class="Cate-SubCateName">'.$subcategory->category.'</div>  <div class="Cateactions"> <div class="btn-group Rtact">';
                                if($user->can('category-edit')){
                                        // $temp_html.='<a href="' . url("/sub-category/edit/".$subcategory->id) . '" title="Edit" ><i class="fas fa-edit"></i></a>';
                                        $temp_html.='<a href="' . route("categories.edit", $subcategory->id). '" title="Edit" ><i class="fas fa-edit"></i></a>';
                                    }
                                    if($user->can('category-delete')){
                                        $temp_html.='<button type="button" class="btn btn-xs btn-flat show_confirm delete-btn" data-bs-toggle="modal" data-bs-target="#deleteModal'. $subcategory->id .'" data-category-id="'. $subcategory->id .'"><i class="fa fa-trash-o" aria-hidden="true"> </i></button>';
                                        $temp_html.='<div class="modal fade" id="deleteModal'. $subcategory->id .'" tabindex="-1" aria-labelledby="deleteModalLabel" aria-hidden="true">
                                        <div class="modal-dialog">
                                        <div class="modal-content">
                                        <div class="modal-header">
                                            <h5 class="modal-title" id="deleteModalLabel">Delete Confirmation</h5>
                                            </div>
                                            <div class="modal-body">
                                            Are you sure you want to delete this category?
                                            </div>
                                            <div class="modal-footer">
                                            <button type="button" class="btn btn-secondary modal-btn" data-dismiss="modal">Cancel</button>
                                            <form method="POST" action="'.route("categories.destroy", $subcategory->id).'">
                                            <input name="_method" type="hidden" value="DELETE">
                                            <input type="hidden" name="_token" value="'.csrf_token().'" />
                                                <button type="submit" class="btn btn-danger text-right modal-btn">Delete</button>
                                            </form>
                                            </div>
                                        </div>
                                        </div>
                                        </div>';
                                    }
                            $temp_html    .= '</div></div>';
                            $temp_html  .= '</div>'; 
                        }
                        $temp_html  .= '</div>';   

                    $temp_html      .=   '</div>';
                    //                     </div>';
                    return $temp_html;
                }
            });

            $datatables = $datatables->addColumn('actions', function ($category) {
                $user               = Auth()->user();
                $actions            ='<div class="action-icons">';
                // $actions            .= '<a href="' . route("categories.show", $category->id) . '" title="Show" ><i class="fas fa-eye"></i></a>';
                if($user->can('category-edit')){
                    $actions.='<a href="' . route("categories.edit", $category->id) . '" title="Edit" ><i class="fas fa-edit"></i></a>';
                }
                // if($user->can('category-create')){
                //     $actions.='<a href="'.url('/sub-category/create/'.$category->id).'" title="Add Sub Category" ><i class="fas fa-plus"></i></a>';
                // }
                if($user->can('category-delete')){
                //     $actions.='<form method="POST" action="'.route("categories.destroy", $category->id).'">
                //     <input name="_method" type="hidden" value="DELETE">
                //     <input type="hidden" name="_token" value="'.csrf_token().'" />
                //     <button onclick="return confirm(\''."Are you sure to delete this category ?".'\')" type="submit" class="btn btn-xs btn-flat show_confirm" data-toggle="tooltip" title="Delete"> <i class="fa fa-trash-o" aria-hidden="true"> </i></button>
                // </form>';
                    if($category->subCategory->isEmpty()){
                        $confirmation_msg="Are you sure you want to delete this category?";
                    }
                    else {
                        $confirmation_msg="Sorry, this category has subcategories. If you want to delete this category it will delete subcategories also. Do you want to proceed?";
                    }

                    $actions.='<button type="button" class="btn btn-xs btn-flat show_confirm delete-btn" data-bs-toggle="modal" data-bs-target="#deleteModal'. $category->id .'" data-category-id="'. $category->id .'"><i class="fa fa-trash-o" aria-hidden="true"> </i></button>';
                    
                    
                    $actions.='<div class="modal fade" id="deleteModal'. $category->id .'" tabindex="-1" aria-labelledby="deleteModalLabel" aria-hidden="true">
                    <div class="modal-dialog">
                    <div class="modal-content">
                    <div class="modal-header">
                        <h5 class="modal-title" id="deleteModalLabel">Delete Confirmation</h5>
                        </div>
                        <div class="modal-body">
                        '.$confirmation_msg.'
                        </div>
                        <div class="modal-footer">
                        <button type="button" class="btn btn-secondary modal-btn" data-dismiss="modal">Cancel</button>
                        <form method="POST" action="'.route("categories.destroy", $category->id).'">
                        <input name="_method" type="hidden" value="DELETE">
                        <input type="hidden" name="_token" value="'.csrf_token().'" />
                            <button type="submit" class="btn btn-danger text-right modal-btn">Delete</button>
                        </form>
                        </div>
                    </div>
                    </div>
                    </div>';
                }
                $actions.='</div>';
                return $actions;


            });

            $datatables = $datatables->escapeColumns([])->toJson();
                return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController categoryList :- " . $e->getMessage());
        } 


    }

   





   public function subCategoryCreate($id)
    {
        try {
            $category       = Categories::find($id);
            if(!$category){
                return Redirect::back();
            }
            return view('categories.sub-create',compact('category'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController create :- " . $e->getMessage());
        } 
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function subCategoryStore(Request $request)
    {
        // try {
            $this->validate($request, [
                'parent_category'   => 'required',
                'category' => 'required',
            ]);
            //dd($request->all());
            // $categories = Categories::create(['category' => $request->input('category')]);
            
            $request->input('category');

            $categories = new Categories;
            $categories->category   = $request->input('category');
            $categories->company_id = Session::get('company');
            $categories->parent_id  = $request->input('parent_category');;
            $categories->save();

            return redirect()->route('categories.index')
                            ->with('success','Category created successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController store :- " . $e->getMessage());
        // } 
    
    }
    
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function subCategoryShow($id)
    {
        try {
            $category = Categories::find($id);
            return view('categories.show',compact('category'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController show :- " . $e->getMessage());
        } 
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function subCategoryEdit($id)
    {
        try {
            $category = Categories::with('parentCategory')->find($id);
            return view('categories.sub-edit',compact('category'));
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController edit :- " . $e->getMessage());
        } 
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function subCategoryUpdate(Request $request, $id)
    {
        // try {
            $this->validate($request, [
                'category' => 'required',
            ]);
        
            $categories = Categories::find($id);
            $categories->category = $request->input('category');
            $categories->company_id = Session::get('company');
            $categories->save();

            return redirect()->route('categories.index')
                            ->with('success','Category updated successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in CategoriesController update :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function subCategoryDestroy($id)
    {
        try {
            DB::table("categories")->where('id',$id)->delete();
                return redirect()->route('categories.index')
                        ->with('success','Category deleted successfully');
        }
        catch (\Exception $e) {
            Log::error("Error in CategoriesController destroy :- " . $e->getMessage());
        } 
    }


}