<?php
    
namespace App\Http\Controllers;
    
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Company;
use App\Models\CompanyUser;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Traits\HasRoles;
use DB;
use Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
use Auth;

    
class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    function __construct()
    {
         $this->middleware('permission:user-list|user-create|user-edit|user-delete', ['only' => ['index','store']]);
         $this->middleware('permission:user-create', ['only' => ['create','store']]);
         $this->middleware('permission:user-edit', ['only' => ['edit','update']]);
         $this->middleware('permission:user-delete', ['only' => ['destroy']]);
    }
    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        try {
            if($request['company_id']==null) {
                    $company_id=0;
                }
            else {
                $company_id=$request['company_id'];
                }

            // return gettype($company_id);
            
            // $data = User::where('deleted',0)->orderBy('id','ASC')->get();
            // // return Datatables::of($data)
            // //             ->make(true);
            // // echo "<pre>";
            // // dd($data->toSql());exit;
        
            // return view('users.index',compact('data'));
            return view('users.index',compact('company_id'));
        }
        catch (\Exception $e) {
            Log::error("Error in UserController index :- " . $e->getMessage());
        } 
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        try {
            $roles = Role::where('id','!=','1')->get();

            $companies = Company::pluck('company_name','id')->all();
            // $companies = DB::table('company')->selct('company_name')->all();
            // return $companies;
        
            return view('users.create',compact('roles','companies'));
        }
        catch (\Exception $e) {
            Log::error("Error in UserController create :- " . $e->getMessage());
        } 
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // try {
            $this->validate($request, [
                'name' => 'required',
                'email' => 'required|email|unique:users,email',
                'phone' => 'required',
                'password' => 'required|same:confirm-password',
                'roles' => 'required',
                // 'companies' => 'required'
            ]);
        
            $input = $request->all();
            $input['password'] = Hash::make($input['password']);
        
            $user = User::create($input);
            $user->assignRole($request->input('roles'));

            if(Auth::user()->id == 1){
                $company_array = $request->companies;
            }
            else {
                $company_array = CompanyUser::where('user_id', Auth::user()->id)->pluck('company_id')->all();
                // return $company_array;
            }

            // $companies = new Company;
            // foreach($request->companies as $item => $value) {
            //     $company_id = $companies->getCompanyIdByCode($value);
            //     $data[$value]=array(
            //         'company_id'=>$company_id,
            //         'user_id'=>$user->id,
            //     );
            // }

            foreach($company_array as $item => $value) {
                $data[$value]=array(
                    'company_id'=>(int)$value,
                    'user_id'=>$user->id,
                );
            }

            CompanyUser::insert($data);
        
            return redirect()->route('users.index')
                            ->with('success','User created successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in UserController store :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $user = User::find($id);
            return view('users.show',compact('user'));
        }
        catch (\Exception $e) {
            Log::error("Error in UserController show :- " . $e->getMessage());
        } 
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        try {
            $user = User::find($id);
            $roles = Role::where('id','!=','1')->get();
            // $userRole = $user->roles->pluck('name','name')->all();
            $userRole = $user->roles->pluck('id','id')->all();
            $company = $user->userCompany->pluck('company_id','company_id');

            $companies = Company::pluck('company_name','id')->all();

            return view('users.edit',compact('user','companies','company','roles','userRole'));
        }
        catch (\Exception $e) {
            Log::error("Error in UserController edit :- " . $e->getMessage());
        } 
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        // try {
            $this->validate($request, [
                'name' => 'required',
                'email' => 'required|email|unique:users,email,'.$id,
                'phone' => 'required',
                'password' => 'same:confirm-password',
                'roles' => 'required',
                // 'companies' => 'required'
            ]);
        
            $input = $request->all();
            if(!empty($input['password'])){ 
                $input['password'] = Hash::make($input['password']);
            }else{
                $input = Arr::except($input,array('password'));    
            }
        
            $user = User::find($id);
            $user->update($input);
            DB::table('model_has_roles')->where('model_id',$id)->delete();
        
            $user->assignRole($request->input('roles'));

            DB::table('company_users')->where('user_id',$id)->delete();

            $company_array = $request->companies;

            foreach($company_array as $item => $value) {
                $data[$value]=array(
                    'company_id'=>(int)$value,
                    'user_id'=>$user->id,
                );
            }

            CompanyUser::insert($data);

        
            return redirect()->route('users.index')
                            ->with('success','User updated successfully');
        // }
        // catch (\Exception $e) {
        //     Log::error("Error in UserController update :- " . $e->getMessage());
        // } 
    }
    
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            // $user=User::find($id);
            // if($user) {
            //     $user->deleted = 1;
            //     $user->save();
            // }

            // return redirect()->route('users.index')
            //                 ->with('success','User deleted successfully');
            $model = User::find( $id );
            $model->delete();
            return redirect()->route('users.index')
                        ->with('success','User deleted successfully');
        }
        catch (\Exception $e) {
            Log::error("Error in UserController destroy :- " . $e->getMessage());
        } 
    }

    public function userList(Request $request)
    {
        try {
            $User = new User;
            $company_id = $request['company_id'];

                
            //retrieve user details
            $result = $User->get_details_user($company_id);
            // $result = $User->get_details_user($company_id);
            $data = collect($result);

            // echo "<pre>";
            // print_r($data);exit;

            // $data = User::where('deleted',0)->orderBy('id','ASC');
            // return Datatables::of($data)
            //             ->make(true);
            $datatables = datatables()->of($data)->addColumn('sl_no', function () {
                return '';
            });

            // $datatables = $datatables->addColumn('name', function ($data) {
            //     $user_name = $data->name;
            //     return $user_name;
            // });

            // $datatables = $datatables->addColumn('email', function ($data) {
            //     $user_email = $data->email;
            //     return $user_email;
            // });

            // $datatables = $datatables->addColumn('phone', function ($data) {
            //     $user_email = $data->phone;
            //     return $user_phone;
            // });

            // $datatables = $datatables->addColumn('roles', function ($data) {
            //     if(count($data)) {
            //     foreach($data as $key => $user) {
            //     $roles="";
            //     if(!empty($user->getRoleNames())) {
            //         foreach($user->getRoleNames() as $v) {
            //         $roles.="<label class='badge badge-success'>{{".$v."}}</label>";
            //         }
            //     }
            //     return $roles;
            // }
            // }
            // });

            $datatables = $datatables->addColumn('roles', function ($data) {
                // $user = new User;
                $roles="";
                $user=User::find($data->id);
                // $user=$data->id;
                if($user) {
                    if(!empty($user->getRoleNames())) {
                        foreach($user->getRoleNames() as $v) {
                        $roles.="<label class='badge badge-warning'> ".$v." </label>&nbsp;&nbsp;";
                        }
                    }
                }
                return $roles;
            });

            $datatables = $datatables->addColumn('company', function ($data) {
                $message="";
                $companies = DB::table('company_users')->where('user_id',$data->id)
                                ->select('company_id','user_id')->first();

                if($companies!=null)
                {
                    $company=Company::find($companies->company_id);
                    $message.=$company->company_name;
                    // $message.="<label class='badge badge-warning'> ".$company->company_name." </label>&nbsp;&nbsp;";
                }
                return $message;
            });


            $datatables = $datatables->addColumn('actions', function ($data) {
                $user = Auth()->user();
                $actions='<div class="action-icons">';
                if($user->can('user-edit')){
                    $actions.='<a href="' . route("users.edit", $data->id) . '" title="Edit" ><i class="fas fa-edit"></i></a>&nbsp;';
                }
                if($user->can('user-delete')){
                    $actions.='<form method="POST" action="'.route("users.destroy", $data->id).'">
                    <input name="_method" type="hidden" value="DELETE">
                    <input type="hidden" name="_token" value="'.csrf_token().'" />
                    <button onclick="return confirm(\''."Are you sure to delete this user ?".'\')" type="submit" class="btn btn-xs btn-flat show_confirm" data-toggle="tooltip" title="Delete"> <i class="fa fa-trash"> </i></button>
                    </form>';
                }
                $actions.='</div>';
                return $actions;

                // $user_role = $data->role_name;
                // return $user_role;
            });

            $datatables = $datatables->escapeColumns([])->toJson();
                return $datatables;
        }
        catch (\Exception $e) {
            Log::error("Error in UserController userList :- " . $e->getMessage());
        } 


    }
    


}

