<?php

namespace App\Models;

use DB;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;
use Auth;
use Illuminate\Support\Collection;

class Categories extends Model {

    /**
     * The database table used by the model.
     *
     * @var string
     */
    protected $table = 'categories';
    protected $primaryKey = 'id';

    protected $fillable = [
        'category',
    ];

    /**
     * Function for inserting tender attachment in the table
     * @param string $fileName
     * @param int $last_tender_id
     * @return int
     */
    // public static function getAllCountryName() {
    //     return DB::table('categories')->lists('category','category');
    // }

    public $timestamps = false;

    public function getAllCategories($company_id) {
        try {
                $category =  DB::table('categories')->where('company_id',$company_id)->orderBy('id', 'DESC');
            return $category->get();
        } catch (\Exception $e) {
            Log::error("SQL error in Categories get_categories :- " . $e->getMessage());
        }
    }

    public function getCategoryIdByName($name) {
        $categories = DB::table('categories')->where('category',$name)->first();
        return $categories->id;
    }
    
    public function subCategory(){
        return $this->hasMany(self::class,'parent_id','id');
    }

    public function parentCategory(){
        return $this->belongsTo(self::class,'parent_id');
    }

    public function company()
    {
        return $this->belongsTo(Company::class);
    }

    public function products()
    {
        return $this->hasMany(Product::class,'category_id','id' );
    }

    public function subCategoryProducts()
    {
        return $this->hasMany(Product::class,'subcategory_id','id' );
    }

    public function buildTree(Collection $elements, $parentId = null) {
        $branch = [];
    
        foreach ($elements as $element) {
            if ($element->parent_id === $parentId) {
                $children = buildTree($elements, $element->id);
    
                if ($children) {
                    $element->children = $children;
                }
    
                $branch[] = $element;
            }
        }
    
        return collect($branch);
    }

    public function productCategory() {
        return $this->hasMany(Product::class,'category_id');
    }

    public function productSubcategory() {
        return $this->hasMany(Product::class,'subcategory_id');
    }
}