<?php
  
namespace App\Models;
 
// use App\Models\CompanyCategories;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;
use Spatie\Permission\Traits\HasRoles;
use DB;
use Str;
use Laravel\Sanctum\NewAccessToken;
use Illuminate\Support\Facades\Log;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

  
class Company extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable, HasRoles, SoftDeletes;
  
    /**
     * The attributes that are mass assignable.
     *
     * @var array

     */

    protected $table = 'company';
    protected $primaryKey = 'id';


    protected $fillable = [
        'company_name',
        'company_code',
        'building_name',
        'area',
        'landmark',
        'town',
        'pin',
        'state',
        'country',
        'email',
        'website',
        'telephone',
        // 'note'
        
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array
     */
    protected $hidden = [
   
    ];

    public function getDetailsCompany() {
        try {

            $company = Company::select('id', 'company_name', 'company_code', 'building_name', 'area', 'landmark', 'town', 'pin', 'state', 'country',
                    'email', 'telephone', 'website', 'note')
                    ->orderBy('id', 'DESC');


           return $company->get();
        } catch (\Exception $e) {
            Log::error("SQL error in Company getDetailsCompany :- " . $e->getMessage());
        }
    }

    public function getReportsCompany($company_type="",$start_date="", $end_date="") {
        try {

            $company = Company::select('id', 'company_name', 'company_code', 'company_type', 'building_name', 'area', 'landmark', 'town', 'pin', 'state', 'country',
                    'email', 'telephone', 'website', 'note','created_at')
                    ->orderBy('id', 'DESC');
            if($company_type!="") {
                $company = $company->where('company_type',$company_type);
            }
            if($start_date!="") {
                $company = $company->where('created_at','>=',$start_date);
            }
            if($end_date!="") {
                $company = $company->where('created_at','<=',$end_date);
            }

           return $company->get();
        } catch (\Exception $e) {
            Log::error("SQL error in Company getDetailsCompany :- " . $e->getMessage());
        }
    }


    public function getCategoryNames($id) {
        try {

            $company = DB::table('company_categories')
                            ->join('categories', 'categories.id', '=', 'company_categories.category_id')  
                            ->where('company.id','!=',$id)            
                            ->select('categories.category as category_name');

           return $company->get();
        } catch (\Exception $e) {
            Log::error("SQL error in Company getDetailsCompany :- " . $e->getMessage());
        }
    }

    public function companyCategories() {
        return $this->hasMany(CompanyCategory::class);
    }

    public function getCompanyIdByCode($name) {
        $companies = DB::table('company')->where('company_code',$name)->first();
        return $companies->id;
        
    }

}
